/** decode interface.cpp : Defines the decoding part of application.

you have to include in the include directories \ffmpeg-git-xxxxxxx-dev\include
and the path to the library files (ffmpeg-git-xxxxxx-dev\lib) in the library directory paths.
You also have to link to avcodec.lib, swscale.lib, avformat.lib and avutil.lib.
When updating ffmpeg, simply update the include and library directory paths in 
the prefs. To facilitate this, there're macros in the property sheets so that you only have
to update the macro and all the directories will be updated. See "MyMacros", "MyMacrosX86", 
and "MyMacrosX64". */


#include "stdafx.h"
#include "decode interface.h"
#include "ffi ffmpeg Interface.h"
extern "C" {
#include "libavformat\avformat.h"
#include "libswscale\swscale.h"
#include "libavutil\imgutils.h"
}


///////////////////////////
// static Variables
#ifdef _DEBUG
/** A file that is opened when the s_cDecodeInterface is constructed and is used throughout the code to log stuff.
	It gets closed when dll is unloaded. It's not thread safe when used with multiple open video files
	simultaneously. */
static FILE*			s_pfOutFile= NULL;
#endif
/** Single object which stores every decode file object that gets created. Is created upon loading DLL in process
	and closes and also closes all open files upon destruction when dll is unloading. */
static CDecodeInterface	s_cDecodeInterface;



/////////////////////////////////////////////////////////////////////////////////////
// Decode API, calls the methods on the decode interface object, s_cDecodeInterface.
extern "C" FFMPEGINTERFACE_API int __stdcall DecoderOpenFile(const char szFilename[], 
	PixelFormat aePixFmtPreferred[], int nFmts, SDecodeFileParams* psDecodeFileParams)
{
	return s_cDecodeInterface.OpenFile(szFilename, aePixFmtPreferred, nFmts, psDecodeFileParams);
}

extern "C" FFMPEGINTERFACE_API int __stdcall DecoderGetNextFrame(int nFileId, unsigned char auchBuffer[], 
	int nSizeIn, double* pdPts)
{
	return s_cDecodeInterface.GetNextFrame(nFileId, auchBuffer, nSizeIn, pdPts);
}

extern "C" FFMPEGINTERFACE_API int __stdcall DecoderSeekToTimestamp(int nFileId, double dTimestamp)
{
	return s_cDecodeInterface.SeekToTimestamp(nFileId, dTimestamp);
}

extern "C" FFMPEGINTERFACE_API void __stdcall DecoderCloseFile(int nFileId)
{
		s_cDecodeInterface.CloseFile(nFileId);
}

extern "C" FFMPEGINTERFACE_API int __stdcall DecoderGetNumFiles()
{
	return s_cDecodeInterface.GetNumFiles();
}

extern "C" FFMPEGINTERFACE_API void __stdcall DecoderCloseAllFiles()
{
	s_cDecodeInterface.CloseAllFiles();
}

extern "C" FFMPEGINTERFACE_API int __stdcall DecoderPeekPicFmt(const char szFilename[])
{
	if (!szFilename)
		return -1;
	// av_register_all() is called in the constructor of the s_cDecodeInterface object
	AVFormatContext *pFormatCtx= NULL;
	// Open the video file
	if(avformat_open_input(&pFormatCtx, szFilename, NULL, NULL)!=0)
	{
		av_close_input_file(pFormatCtx);
		return -1;
	}

	// Populate stream info.
	if(avformat_find_stream_info(pFormatCtx, NULL)<0)
	{
		av_close_input_file(pFormatCtx);
		return -1;
	}

	// Find the best video stream.
	AVCodec* pCodec= NULL;
	int nStream= av_find_best_stream(pFormatCtx, AVMEDIA_TYPE_VIDEO, -1, -1, &pCodec, 0);
	if(nStream<0)
	{
		av_close_input_file(pFormatCtx);
		return -1;
	}

	// Get pix fmt for this stream.
	int nFmt= pFormatCtx->streams[nStream]->codec->pix_fmt;
	av_close_input_file(pFormatCtx);
	return nFmt;
}



/////////////////////////////////////////
// Container for all decoding files, implementation
CDecodeInterface::CDecodeInterface()
{
	InitializeCriticalSection(&m_rLock);
	m_nIndex= 0;
	m_cFileList.clear();
	av_register_all();	// So we can access all the decoding ffmpeg funcs.
#ifdef _DEBUG
	if (!s_pfOutFile)
		fopen_s(&s_pfOutFile, "Default FFmpeg DLL Decoder Interface log.txt", "a");
#endif
}

CDecodeInterface::~CDecodeInterface()
{
	CloseAllFiles();	// Make sure all files are closed
	DeleteCriticalSection(&m_rLock);
#ifdef _DEBUG
	fclose(s_pfOutFile);
	s_pfOutFile= NULL;
#endif
}

int CDecodeInterface::OpenFile(const char szFilename[], PixelFormat aePixFmtPreferred[], int nFmts, 
	SDecodeFileParams* psDecodeFileParams)
{
	if (!psDecodeFileParams)	// So we can use nFileId for sure
		return BAD_PARAMS;

	CDecodeFile* pcDecodeFile= NULL;
	EnterCriticalSection(&m_rLock);	// Protect access to file list
	pcDecodeFile= new CDecodeFile(m_nIndex); // Give it a unique identifier
	m_cFileList.push_back(pcDecodeFile);
	psDecodeFileParams->nFileId= m_nIndex++;
	LeaveCriticalSection(&m_rLock);
	int nRes;
	if ((nRes= pcDecodeFile->OpenFile(szFilename, aePixFmtPreferred, nFmts, psDecodeFileParams))!=0)
	{
		CloseFile(psDecodeFileParams->nFileId);	// File had trouble so close it
		psDecodeFileParams->nFileId= -1;
	}
	return nRes;
}

CDecodeFile* CDecodeInterface::GetCDecodeFile(int nFileId)
{
	CDecodeFile* pcDecode= NULL;	// So we return NULL if not found, and so function supports a invalid nFileId
	EnterCriticalSection(&m_rLock);
	for (m_cIterator= m_cFileList.begin();m_cIterator!=m_cFileList.end();++m_cIterator)
	{
		if (nFileId==(*m_cIterator)->m_nIndex)
		{
			pcDecode= *m_cIterator;	// Found match
			break;
		}
	}
	LeaveCriticalSection(&m_rLock);
	return pcDecode;
}

int CDecodeInterface::GetNextFrame(int nFileId, unsigned char auchBuffer[], int nSizeIn, double* pdPts)
{
	CDecodeFile* pcDecode= GetCDecodeFile(nFileId);	// Returns NULL if not found
	return pcDecode?pcDecode->GetNextFrame(auchBuffer, nSizeIn, pdPts):NO_SUCH_OBJECT;	// If not found file return error.
}

int CDecodeInterface::SeekToTimestamp(int nFileId, double dTimestamp)
{
	CDecodeFile* pcDecode= GetCDecodeFile(nFileId);	// Returns NULL if not found
	return pcDecode?pcDecode->SeekToTimestamp(dTimestamp):NO_SUCH_OBJECT;	// If not found file return error.
}

void CDecodeInterface::CloseFile(int nFileId)
{
	CDecodeFile* pcDecode= NULL; // Assume file doesn't exist, support invalid nFileId.
	EnterCriticalSection(&m_rLock);
	for (m_cIterator= m_cFileList.begin();m_cIterator!=m_cFileList.end();++m_cIterator)
	{
		if (nFileId==(*m_cIterator)->m_nIndex)	// Found?
		{
			pcDecode= *m_cIterator;	// Get file
			m_cFileList.erase(m_cIterator);	// Remove from list
			break;
		}
	}
	LeaveCriticalSection(&m_rLock);
	// Delete outside of critical section so as to not hold up everyone.
	delete pcDecode;	// Delete file (and close it), if NULL no effect.
	pcDecode= NULL;
}

void CDecodeInterface::CloseAllFiles()
{
	EnterCriticalSection(&m_rLock);	// Just in case
	for (m_cIterator= m_cFileList.begin();m_cIterator!=m_cFileList.end();)
	{
		delete *m_cIterator;	// Delete current file
		m_cIterator= m_cFileList.erase(m_cIterator);	// Move iterator to next file

	}
	LeaveCriticalSection(&m_rLock);
}

int CDecodeInterface::GetNumFiles()
{
	int nRes;
	EnterCriticalSection(&m_rLock);
	nRes= m_cFileList.size();
	LeaveCriticalSection(&m_rLock);
	return nRes;
}



///////////////////////////////////////////////////////
// Actual decoding file implementation
CDecodeFile::CDecodeFile(int nIndex):m_nIndex(nIndex)
{
	InitializeCriticalSection(&m_rFileSafe);
	m_pFormatCtx= NULL;	// Make sure you start with all pointers nulled
	m_pCodecCtx= NULL;
	m_pCodec= NULL;

	m_pFrame= NULL;
	m_pFrameOut= NULL;
	m_pConvertCtx= NULL;
	m_ePixFmtOut= PIX_FMT_NONE;

	m_pAVPacket= new AVPacket();
	
	m_nBufferSize= 0;

	m_nVideoStream= -1;	// No stream

	m_bDone= false;	// So that we'll start reading packets.
	m_dPts= 0;
}

CDecodeFile::~CDecodeFile()
{
	if (m_pAVPacket)
	{
		delete m_pAVPacket;
		m_pAVPacket= NULL;
	}
	FinishUp();	// Close the file
	DeleteCriticalSection(&m_rFileSafe);
}

void CDecodeFile::FinishUp()
{
	// Close codecs
	if (m_pCodecCtx)
	{
		avcodec_close(m_pCodecCtx);
		m_pCodecCtx= NULL;
		m_pCodec= NULL;	// No need to close
	}
	// Close format context. Must not be called before closing m_pCodecCtx.
	if (m_pFormatCtx)
	{
		av_close_input_file(m_pFormatCtx);
		m_pFormatCtx= NULL;
	}
	// Close frames that stored incoming pics.
	if (m_pFrame)
	{
		av_free(m_pFrame);	// We didn't allocate buffers so we don't close it
		m_pFrame= NULL;
	}
	if (m_pFrameOut)
	{
		if (m_pFrameOut->data[0])	// Delete the image buffers, allocated by us.
			av_freep(&m_pFrameOut->data[0]);
		av_free(m_pFrameOut);	// Actual image
		m_pFrameOut= NULL;
	}

	// Context that converted between different image formats
	if (m_pConvertCtx)
	{
		sws_freeContext(m_pConvertCtx);
		m_pConvertCtx= NULL;
	}

	m_nVideoStream= -1;	// No stream

	m_ePixFmtOut= PIX_FMT_NONE;	// Clear to fresh.
	m_nBufferSize= 0;
	m_bDone= false;

	m_dPts= 0;
}

int CDecodeFile::OpenFile(const char szFilename[], PixelFormat ePixFmtPreferred[], int nFmts, 
	SDecodeFileParams* psDecodeFileParams)
{
	if ( !szFilename || !psDecodeFileParams)	// Ok if ePixFmtPreferred is null
		return BAD_PARAMS;


	EnterCriticalSection(&m_rFileSafe);
	if (m_pFormatCtx)	// Make sure we don't open twice the same file, it's pointing somewhere if opened before.
	{
		LeaveCriticalSection(&m_rFileSafe);
		return FILE_OPEN;	// File was already open
	}

	// Open the video file
	if(avformat_open_input(&m_pFormatCtx, szFilename, NULL, NULL)!=0)
	{
		FinishUp();	// Clear to fresh on error
		LeaveCriticalSection(&m_rFileSafe);
		return D_CANT_OPEN_FILE;
	}

	// Populate stream info.
	if(avformat_find_stream_info(m_pFormatCtx, NULL)<0)
	{
		FinishUp();
		LeaveCriticalSection(&m_rFileSafe);
		return D_CANT_READ_FILE;
	}

	// Find the best video stream and get codec.
	m_nVideoStream= av_find_best_stream(m_pFormatCtx, AVMEDIA_TYPE_VIDEO, -1, -1, &m_pCodec, 0);
	if(m_nVideoStream<0)
	{
		FinishUp();
		LeaveCriticalSection(&m_rFileSafe);
		return D_CANT_READ_FILE; // Didn't find a video stream
	}

	// Get a pointer to the codec context for the video stream, context was created above internally
	m_pCodecCtx= m_pFormatCtx->streams[m_nVideoStream]->codec;

#ifdef _DEBUG
	fprintf(s_pfOutFile, "-----------------------Format Context-------------------------------\n");
	fprintf(s_pfOutFile, "%d, Name: %s\n", GetTickCount(), m_pFormatCtx->av_class->class_name);
	fprintf(s_pfOutFile, "%d, Bitrate: %d\n", GetTickCount(), m_pFormatCtx->bit_rate);
	fprintf(s_pfOutFile, "%d, Start time: %I64d\n", GetTickCount(), m_pFormatCtx->start_time);
	fprintf(s_pfOutFile, "%d, First packet offset: %I64d\n", GetTickCount(), m_pFormatCtx->data_offset);
	fprintf(s_pfOutFile, "%d, Duration: %I64d\n", GetTickCount(), m_pFormatCtx->duration);
	fprintf(s_pfOutFile, "%d, Filesize: %I64d\n", GetTickCount(), m_pFormatCtx->file_size);
	fprintf(s_pfOutFile, "%d, Format Long name: %s\n", GetTickCount(), m_pFormatCtx->iformat->long_name);
	fprintf(s_pfOutFile, "%d, Max analyze duration: %d\n", GetTickCount(), m_pFormatCtx->max_analyze_duration);
	fprintf(s_pfOutFile, "%d, Mux rate: %d\n", GetTickCount(), m_pFormatCtx->mux_rate);
	fprintf(s_pfOutFile, "%d, n Streams: %d\n", GetTickCount(), m_pFormatCtx->nb_streams);
	if (m_pFormatCtx->iformat)
	{
		fprintf(s_pfOutFile, "%d, Format name: %s\n", GetTickCount(), m_pFormatCtx->iformat->name);
		fprintf(s_pfOutFile, "%d, Format Long name: %s\n", GetTickCount(), m_pFormatCtx->iformat->long_name);
	}
	fprintf(s_pfOutFile, "-----------------------Stream---------------------------------------\n");
	fprintf(s_pfOutFile, "%d, Avg FR den: %d\n", GetTickCount(), m_pFormatCtx->streams[m_nVideoStream]->avg_frame_rate.den);
	fprintf(s_pfOutFile, "%d, Avg FR num: %d\n", GetTickCount(), m_pFormatCtx->streams[m_nVideoStream]->avg_frame_rate.num);
	fprintf(s_pfOutFile, "%d, Real FR den: %d\n", GetTickCount(), m_pFormatCtx->streams[m_nVideoStream]->r_frame_rate.den);
	fprintf(s_pfOutFile, "%d, Real FR num: %d\n", GetTickCount(), m_pFormatCtx->streams[m_nVideoStream]->r_frame_rate.num);
	fprintf(s_pfOutFile, "%d, Time base num: %d\n", GetTickCount(), m_pFormatCtx->streams[m_nVideoStream]->time_base.num);
	fprintf(s_pfOutFile, "%d, Time base den: %d\n", GetTickCount(), m_pFormatCtx->streams[m_nVideoStream]->time_base.den);
	fprintf(s_pfOutFile, "%d, Start T: %I64d\n", GetTickCount(), m_pFormatCtx->streams[m_nVideoStream]->start_time);
	fprintf(s_pfOutFile, "%d, N frames: %I64d\n", GetTickCount(), m_pFormatCtx->streams[m_nVideoStream]->nb_frames);
	fprintf(s_pfOutFile, "%d, Cur dts: %I64d\n", GetTickCount(), m_pFormatCtx->streams[m_nVideoStream]->cur_dts);
	fprintf(s_pfOutFile, "%d, Ref dts: %I64d\n", GetTickCount(), m_pFormatCtx->streams[m_nVideoStream]->reference_dts);
	fprintf(s_pfOutFile, "-----------------------Codec Context--------------------------------\n");
	fprintf(s_pfOutFile, "%s\n", m_pCodec->long_name);
	fprintf(s_pfOutFile, "%s\n", m_pCodec->name);
	fprintf(s_pfOutFile, "%d, ID: %d\n", GetTickCount(), m_pCodecCtx->codec_id);
	fprintf(s_pfOutFile, "%d, SPS: %d\n", GetTickCount(), m_pCodecCtx->sample_rate);
	fprintf(s_pfOutFile, "%d, nChannels: %d\n", GetTickCount(), m_pCodecCtx->channels);
	fprintf(s_pfOutFile, "%d, time_base den: %d\n", GetTickCount(), m_pCodecCtx->time_base.den);
	fprintf(s_pfOutFile, "%d, time_base num: %d\n", GetTickCount(), m_pCodecCtx->time_base.num);
	fprintf(s_pfOutFile, "%d, tics_per_frmae: %d\n", GetTickCount(), m_pCodecCtx->ticks_per_frame);
	/*for (int i= 0;m_pCodec->profiles && m_pCodec->profiles[i].profile!=FF_PROFILE_UNKNOWN;++i)
		fprintf(s_pfOutFile, "%s; ", m_pCodec->profiles[i].name);
	fprintf(s_pfOutFile, "\n");*/
#endif
	
	// Initialize codec ctx to use codec
	if(avcodec_open2(m_pCodecCtx, m_pCodec, NULL)<0)
	{
		FinishUp();
		LeaveCriticalSection(&m_rFileSafe);
		return D_CANT_OPEN_DECODER; // Could not open codec
	}

	// Find format of output pic (pic to send back to user)
	if (!ePixFmtPreferred || nFmts<=0 )	// If there's no preffered output format use stream format
	{
		m_ePixFmtOut= m_pCodecCtx->pix_fmt;
	} else if (nFmts==1)	// Use preselected format
	{
		m_ePixFmtOut= ePixFmtPreferred[0];
	} else	// Select best one among the selected formats
	{
		m_ePixFmtOut= PIX_FMT_NONE;
		for (int i= 0;i<nFmts;++i)
		{
			// Assume no alpha
			m_ePixFmtOut= avcodec_find_best_pix_fmt2(m_ePixFmtOut, ePixFmtPreferred[i], m_pCodecCtx->pix_fmt, 0, NULL);
		}
		
		if (m_ePixFmtOut==PIX_FMT_NONE)	// Try best, if fails use first in list
		{
			m_ePixFmtOut= ePixFmtPreferred[0];
		}
#ifdef _DEBUG
		fprintf(s_pfOutFile, "Formats; OutFmt: %d, nFmts: %d\n", m_ePixFmtOut, nFmts);
#endif
	}

	// Make sure output is a valid pix fmt
	if (m_ePixFmtOut == PIX_FMT_NONE)
	{
		FinishUp();
		LeaveCriticalSection(&m_rFileSafe);
		return D_NO_FMT;
	}

	// Prepre frame into which packets as they are read will be copied into.
	// No need to allocate buffer for pic data because it's allocated by av_read_frame
	m_pFrame= avcodec_alloc_frame();
	if (!m_pFrame)
	{
		FinishUp();
		LeaveCriticalSection(&m_rFileSafe);
		return ALLOC_MEMORY_ERROR;	// Not enough memory
	}

#ifdef _DEBUG
	fprintf(s_pfOutFile, "InFmt: %d, OutFmt: %d, Width: %d, Height: %d\n", m_pCodecCtx->pix_fmt, m_ePixFmtOut, 
		m_pCodecCtx->width, m_pCodecCtx->height);
#endif

	// If we need to convert pic to different output than the stream pix fmt, create conversion context, frames...
	if (m_ePixFmtOut!=m_pCodecCtx->pix_fmt)
	{
		// Prepare frame where the final image that has been converted to the proper output format is copied into
		m_pFrameOut= avcodec_alloc_frame();
		if(!m_pFrameOut)
		{
			FinishUp();
			LeaveCriticalSection(&m_rFileSafe);
			return ALLOC_MEMORY_ERROR; // Memory trouble
		}

		// Fill in image with buffers, we have to create the buffers ourselves (with this func).
		if (av_image_alloc(m_pFrameOut->data, m_pFrameOut->linesize, m_pCodecCtx->width, 
			m_pCodecCtx->height, m_ePixFmtOut, 1)<0)
		{
			FinishUp();
			LeaveCriticalSection(&m_rFileSafe);
			return ALLOC_MEMORY_ERROR; // Memory trouble
		}

		// This creates the conversion context used to convert between stream fmt to output fmt.
		m_pConvertCtx = sws_getCachedContext(NULL, m_pCodecCtx->width, m_pCodecCtx->height, 
			m_pCodecCtx->pix_fmt, 
			m_pCodecCtx->width, m_pCodecCtx->height, m_ePixFmtOut, SWS_BICUBIC,
			NULL, NULL, NULL);
		if (!m_pConvertCtx)
		{
			FinishUp();
			LeaveCriticalSection(&m_rFileSafe);
			return NO_CONVERT_FMT; // Cannot convert pix formats
		}
	}

	m_bDone= false;	// Initially read packets.
	m_dPts= 0;
	// Populate return struct with values.
	psDecodeFileParams->dDuration= ((double)m_pFormatCtx->duration)/AV_TIME_BASE;
	psDecodeFileParams->nFrames= (int) m_pFormatCtx->streams[m_nVideoStream]->nb_frames;
	psDecodeFileParams->nMeanFrameRateDen= !m_pFormatCtx->streams[m_nVideoStream]->avg_frame_rate.den?
		m_pFormatCtx->streams[m_nVideoStream]->r_frame_rate.den:
		m_pFormatCtx->streams[m_nVideoStream]->avg_frame_rate.den;
	psDecodeFileParams->nMeanFrameRateNom= !m_pFormatCtx->streams[m_nVideoStream]->avg_frame_rate.den?
		m_pFormatCtx->streams[m_nVideoStream]->r_frame_rate.num:
		m_pFormatCtx->streams[m_nVideoStream]->avg_frame_rate.num;
	psDecodeFileParams->ePixFmtOut= m_ePixFmtOut;
	psDecodeFileParams->nWidth= m_pCodecCtx->width;
	psDecodeFileParams->nHeight= m_pCodecCtx->height;
	/* Determine required input buffer size for get next frame to copy the image. We return the compact picture.
	   Switch is required to circumvent bug where avpicture_layout writes by 1024 passed buffer length for these
	   pix fmts. */
	psDecodeFileParams->nBufferSize= avpicture_get_size(m_ePixFmtOut, m_pCodecCtx->width, m_pCodecCtx->height);
	switch (m_ePixFmtOut) {
    case PIX_FMT_RGB8:
    case PIX_FMT_BGR8:
    case PIX_FMT_RGB4_BYTE:
    case PIX_FMT_BGR4_BYTE:
    case PIX_FMT_GRAY8:
        // Include palette for these pseudo-paletted formats
		psDecodeFileParams->nBufferSize+= 1024;
    }
	m_nBufferSize= psDecodeFileParams->nBufferSize;	// Save required size
	// Also get the linesize of all planes to help user
	if (av_image_fill_linesizes(psDecodeFileParams->anLinesizes, m_ePixFmtOut, m_pCodecCtx->width)<0)
	{
		FinishUp();
		LeaveCriticalSection(&m_rFileSafe);
		return LINESIZES_ERROR;
	}
	LeaveCriticalSection(&m_rFileSafe);

	return 0;	// Done
}

int CDecodeFile::GetNextFrame(unsigned char auchBuffer[], int nSizeIn, double* pdPts)
{
	if (!auchBuffer || !pdPts)
		return BAD_PARAMS;

	EnterCriticalSection(&m_rFileSafe);
	if (!m_pFormatCtx)	// Make sure it's a valid object, this is always null if not yet opened
	{
		LeaveCriticalSection(&m_rFileSafe);
		return BAD_OBJECT;
	}

	if (nSizeIn<m_nBufferSize)	// Make sure size is large enough
	{
		LeaveCriticalSection(&m_rFileSafe);
		return BAD_PARAMS;
	}

	int nRes= 0;
	double dPtsI= m_dPts;	// pts of previous frame
	bool bDone= m_bDone;	// Are we reading still or are we flushing frames now? Initially it's false b/c we're still reading
	while(!bDone && av_read_frame(m_pFormatCtx, m_pAVPacket)>=0)	// Stop if found frame, flushing, or end of file
	{
		// Is this a packet from our video stream?
		if(m_pAVPacket->stream_index==m_nVideoStream)
		{
			// Try to decode frame from packet data
			bDone= DecodeFrame(auchBuffer, nSizeIn);	// True if decoded full frame.
		}
		// Free the packet that was allocated by av_read_frame and holds the pic data
		av_free_packet(m_pAVPacket);
	}


	bool bMoreFlush;	// If we're flushing are there any more frames to be flushed? Otherwise seek to start

	// Are we flushing or about to start flushing?
	// bDone is only false here if we reached end of file so we need to start flushing.
	if (!bDone || m_bDone)	// First and subsequent flush
	{
#ifdef _DEBUG
		fprintf(s_pfOutFile, "Flush PTS1: %g\n", m_dPts);
#endif
		m_pAVPacket->data=NULL;	// We need to get the last frames of some formats by calling decode frame with packet with null data.
		m_pAVPacket->size=0;
		m_bDone= true;	// In subsequent calls we'll be flushing the decoder so don't even enter the while loop above
		// Get next frame since we didn't get a frame yet.
		bMoreFlush= DecodeFrame(auchBuffer, nSizeIn);	// Returns false if no more frames to flush and we didn't get a frame, so seek to start.
#ifdef _DEBUG
		fprintf(s_pfOutFile, "Flush PTS2: %g, More? %d\n", m_dPts, bMoreFlush);
#endif
	} else
	{
		bMoreFlush= true;	// Still more frames to get (we aren't even flushing yet) so don't seek back to zero
	}


	// If we reached the last frame above we need to seek back to start and get the first frame since function always returns a frame
	// We went past valid frames so got reset to zero, i.e. 
	if ((m_dPts<dPtsI && dPtsI!=0)	// pts of current jumped to less than before, but not b/c seeking (when seeking dPtsI=0)
		|| !bMoreFlush			// Done with ALL frames
		|| (m_dPts==dPtsI && dPtsI>0))		// repeating frame pts so we're done too (repeating zero doesn't mean that though)
	{
#ifdef _DEBUG	// Why are we seeking
		fprintf(s_pfOutFile, "Seeking to start, PTS: %g, Ipts: %g, !bMoreFlush: %d\n", m_dPts, dPtsI, !bMoreFlush);
#endif
		if (avformat_seek_file(m_pFormatCtx, m_nVideoStream, INT64_MIN, 0, INT64_MAX, 0)>=0)
		{
			avcodec_flush_buffers(m_pCodecCtx);	// Flush so that no buffer confusion
			m_bDone= false;	// We're not done anymore because we started again
			m_dPts= 0;
			nRes= GetNextFrame(auchBuffer, nSizeIn, pdPts);	// Now get first frame in stream
			if (!nRes)	// If successful indicate that we looped back and picture returned is the first frame again.
				nRes= D_LAST_FRAME;
		} else
		{
			avcodec_flush_buffers(m_pCodecCtx);	// Either way flush buffers.
			nRes= D_SEEK_FAILED;
		}
	}

	*pdPts= m_dPts;	// PTS of current image
	LeaveCriticalSection(&m_rFileSafe);
	return nRes;
}

bool CDecodeFile::DecodeFrame(unsigned char auchBuffer[], int nSizeIn)
{
	int nFrameFinished= 0;

	// Decode as much of the frame from packet and copy into internal buffer. This needs to be called until 
	// we get nFrameFinished true, otherwise a full frame hasn't been decoded.
	avcodec_decode_video2(m_pCodecCtx, m_pFrame, &nFrameFinished, m_pAVPacket);
	// Estimate the pts of the frame. Timestamp is converted to time in sec.
	// DO NOT move it into conditional below otherwise, we'd seek to start upon next call to the get next frame function.
	m_dPts = m_pFrame->best_effort_timestamp*av_q2d(m_pFormatCtx->streams[m_nVideoStream]->time_base);
	// Do we need to account for pFrame->repeat_pict?
#ifdef _DEBUG
	fprintf(s_pfOutFile, "Frame PTS: %g, framefinished?: %d\n", m_dPts, nFrameFinished);
#endif
	// Did we get a video frame?
	if(nFrameFinished)
	{
		if (m_ePixFmtOut!=m_pCodecCtx->pix_fmt)	// If we need to convert pic to different output...
		{
			// Convert to the proper output format
			sws_scale(m_pConvertCtx, m_pFrame->data, m_pFrame->linesize, 0, 
				m_pCodecCtx->height, m_pFrameOut->data, m_pFrameOut->linesize);
			// Copy image to the input buffer.
			avpicture_layout((AVPicture*) m_pFrameOut, m_ePixFmtOut, m_pCodecCtx->width, m_pCodecCtx->height, auchBuffer, nSizeIn);
		} else
		{	// Otherwise just copy it
			avpicture_layout((AVPicture*) m_pFrame, m_ePixFmtOut, m_pCodecCtx->width, m_pCodecCtx->height, auchBuffer, nSizeIn);
		}
		return true;	// Frame gotten
	}
	return false;	// No frame gotten
}

int CDecodeFile::SeekToTimestamp(double dTimestamp)
{
	int nError= 0;
	int64_t nTime;

	EnterCriticalSection(&m_rFileSafe);
	if (!m_pFormatCtx)	// File wasn't opened properly.
	{
		LeaveCriticalSection(&m_rFileSafe);
		return BAD_OBJECT;
	}
	// Calculate the time to seek to in terms of stream time base values.
	nTime= (int64_t)floor(dTimestamp/av_q2d(m_pFormatCtx->streams[m_nVideoStream]->time_base));

	// Reset to zero so that upon next call to GetNextFrame we won't end up seeking to start.
	m_dPts= 0;
#ifdef _DEBUG
	fprintf(s_pfOutFile, "Time (stream): %I64d, Time (our): %g\n", nTime, dTimestamp);
#endif
	if (avformat_seek_file(m_pFormatCtx, m_nVideoStream, INT64_MIN, nTime, INT64_MAX, 0)<0)
	{	// If seeking failed try to seek to start but still return an error.
		nError= D_SEEK_RESET;
		if (avformat_seek_file(m_pFormatCtx, m_nVideoStream, INT64_MIN, 0, INT64_MAX, 0)<0)
		{
			nError= D_SEEK_FAILED;	// Seeking completely failed
		}
	}
	avcodec_flush_buffers(m_pCodecCtx);	// Either way flush buffers!
	LeaveCriticalSection(&m_rFileSafe);
	return nError;
}