/** 
	Copyright 2011 CPL Lab, Cornell University

    This file is part of ffmpeg interface.

    ffmpeg interface is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    ffmpeg interface is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with ffmpeg interface.  If not, see <http://www.gnu.org/licenses/>.

	The ffmpeg API is copyrighted by their respective copyright holders. See
	the license folder (should have been included with this file) for the 
	copyright notices.
	*/

/** Defines the decoder function accessed by the API. */



#ifndef __DECODE_INTERFACE_CPL_H__
#define __DECODE_INTERFACE_CPL_H__

#include <windows.h>
#include <list>
extern "C" {
#include "libavutil\pixfmt.h"
}


// Forward declarations
struct AVFormatContext;
struct AVCodecContext;
struct AVCodec;
struct AVFrame;
struct AVPacket;
struct SwsContext;
struct SDecodeFileParams;
class CDecodeFile;


/** A thread safe class which holds a list of all currently open decoder files and can be used 
	to perform operations on these files. It provides access to these files for the DLL API functions. */
class CDecodeInterface
{
public:
	/** Initializes the instance of this class and registers the functions required for decoding/encoding. 
		Note, in debug version it opens a FILE object and stores it into a FILE* variable called s_fOutFile. 
		This should have been defined in the implementation file above the class functions. */
	CDecodeInterface();
	/** Also closes the log file. */
	virtual ~CDecodeInterface();

	/** Opens a video file for decoding. @see DecoderOpenFile. 
		Each file that is opened is initialized with a const int value which is used to 
		identify that object. */
	int OpenFile(const char szFilename[], PixelFormat ePixFmtPreferred[], int nFmts,  
		SDecodeFileParams* psDecodeFileParams);
	/** Returns the next frame in out video position. @see DecoderGetNextFrame. */
	int GetNextFrame(int nFileId, unsigned char auchBuffer[], int nSizeIn, double* pdPts);
	/** Seeks to the given timestamp in the video. @see DecoderSeekToTimestamp. */
	int SeekToTimestamp(int nFileId, double dTimestamp);
	/** Closes previously opened file. @see DecoderCloseFile. */
	void CloseFile(int nFileId);
	/** Returns the number of decoding files currently open. @see DecoderGetNumFiles. */
	int GetNumFiles();
	/** Closes all open video decoding files. @see DecoderCloseAllFiles. */
	void CloseAllFiles();

private:
	/** Thread safe function that searches the decoder file list for a file with a
		matching index to nFileId and returns the pointer to it if found. Otherwise
		it returns NULL. Pointer returned might become invalid if you call close 
		file for this file ID while you're still using it. */
	CDecodeFile* GetCDecodeFile(int nFileId);

	/** List of all the open decoding files. */
	std::list<CDecodeFile*>	m_cFileList;
	/** Protects access to the file list. */
	CRITICAL_SECTION	m_rLock;
	/** The index value that will be assigned to the next file that will be opened. 
		It's then incremented. Index value is not reused until overflow. Access is
		also protected by above critical section. */
	int m_nIndex;
	/** Iterates the file list. */
	std::list<CDecodeFile*>::iterator m_cIterator;
};


/** A thread safe class which houses a single file that is being decoded. */
class CDecodeFile
{
public:
	/** Initializes the object with a const integer, nIndex that can then be used to locate this object
		in a list. To open the file you need to call {@link OpenFile} on this object.*/
	CDecodeFile(int nIndex);
	/** Also closes the file (if opened). */
	virtual ~CDecodeFile();

	/** Opens and associates this object with a particular video file. This creates and initializes all
		the memory and objects needed for decoding. If it was unsuccessful, it'll return an error code, and
		you can try then again to open another video file. However, if it succeeded in opening a video 
		file you won't be able to open another file with this object. @see DecoderOpenFile. */
	int OpenFile(const char szFilename[], PixelFormat ePixFmtPreferred[], int nFmts,
		SDecodeFileParams* psDecodeFileParams);
	/** Returns the next frame based on our current position in the video stream. @see DecoderGetNextFrame.*/
	int GetNextFrame(unsigned char auchBuffer[], int nSizeIn, double* pdPts);
	/** Seeks in the video to the desired timestamp. @see DecoderSeekToTimestamp.*/
	int	SeekToTimestamp(double dTimestamp);

	/** A value that gets assigned to this object so that you can identify the object by this value. */
	const int m_nIndex;
private:
	/** Closes all open files, codecs etc. It clears the object to fresh state.
		Is not protected with critical section so calling func must protect it.*/
	void	FinishUp();
	/** Decodes the last packet that was gotten using av_read_frame and
		outputs the data on success to auchBuffer in the proper output format.
		The image copied back is in compact format.
		Is not protected with critical section so calling func must protect it.
		@return true if a full frame was gotten and copied to buffer, false otherwise. 
			If you're flushing packets and it returns false it means there's nothing left to flush. */
	bool	DecodeFrame(unsigned char auchBuffer[], int nSizeIn);

	/** Holds the format info of the file. */
	AVFormatContext *m_pFormatCtx;
	/** Holds the codec info of the file. */
	AVCodecContext *m_pCodecCtx;
	/** A codec that used to decode the file. */
	AVCodec *m_pCodec;

	/** Frame in which incoming packet data gets stored. */
	AVFrame *m_pFrame;
	/** The frame into which the output image will saved after it was converted. */
	AVFrame *m_pFrameOut;
	/** The format into which the stream frame will be converted to in order to send back to user. */
	PixelFormat	m_ePixFmtOut;
	/** The struct used to convert between the stream and user desired picture formats. */
	SwsContext *m_pConvertCtx;
	/** Size of the buffer required to pass in by user in order for us to be able to copy the image
		of the desired output format into the user supplied buffer. */
	int m_nBufferSize;

	/** The packet struct into which data is saved internally as frames are being read from stream. */
	AVPacket *m_pAVPacket;
	/** The stream number among the streams of the file that we want to decode. */
	int m_nVideoStream;

	/** If we're done reading the file and we now need to flush the decoder to get the remaining pictures. */
	bool m_bDone;
	/** The PTS (presentation time stamp, the time this frame should be presented to the user) of the
		last picture returned relative to the start of the video (seconds). */
	double	m_dPts;

	/** Protects access to this object. */
	CRITICAL_SECTION m_rFileSafe;
};

#endif