/** encode interface.cpp : Defines the encoding part of application.

you have to include in the include directories \ffmpeg-git-xxxxxxx-dev\include
and the path to the library files (ffmpeg-git-xxxxxx-dev\lib) in the library directory paths.
You also have to link to avcodec.lib, swscale.lib, avformat.lib and avutil.lib.
When updating ffmpeg, simply update the include and library directory paths in 
the prefs. To facilitate this, there're macros in the property sheets so that you only have
to update the macro and all the directories will be updated. See "MyMacros", "MyMacrosX86", 
and "MyMacrosX64". */


#include "stdafx.h"
#include "encode interface.h"
#include "ffi ffmpeg Interface.h"
extern "C" {
#include "libavformat\avformat.h"
#include "libswscale\swscale.h"
#include "libavutil\imgutils.h"
#include "libavutil\opt.h"
#include "libavutil\pixdesc.h"
}


///////////////////////////
// static Variables
#ifdef _DEBUG
/** A file that is opened when the s_cEncodeInterface is constructed and is used throughout the code to log stuff.
	It gets closed when dll is unloaded. It's not thread safe when used with multiple open video files
	simultaneously. */
static FILE*			s_pfOutFile= NULL;
#endif
/** Single object which stores every encode file object that gets created. Is created upon loading DLL in process
	and closes and also closes all open files upon destruction when dll is unloading. */
static CEncodeInterface s_cEncodeInterface;



/////////////////////////////////////////////////////////////////////////////////////
// Encode API, calls the methods on the encode interface object, s_cEncodeInterface.
extern "C" FFMPEGINTERFACE_API int __stdcall EncoderOpenFile(const char* szFilename, const SEncodeParams* psEncodeParams, 
	char* szOpt, int* anOptError, int* pnPicSizeInFmt, int anLinesizes[4], int* pnFileID)
{
	return s_cEncodeInterface.OpenFile(szFilename, psEncodeParams, szOpt, anOptError, pnPicSizeInFmt, anLinesizes, pnFileID);
}

extern "C" FFMPEGINTERFACE_API int __stdcall EncoderWriteFrame(unsigned char* auchBuffer, int nSizeIn, int nFileID)
{
	return s_cEncodeInterface.WriteFrame(auchBuffer, nSizeIn, nFileID);
}

extern "C" FFMPEGINTERFACE_API void __stdcall EncoderCloseFile(int nFileID)
{
	s_cEncodeInterface.CloseFile(nFileID);
}

extern "C" FFMPEGINTERFACE_API int __stdcall EncoderGetNumFiles()
{
	return s_cEncodeInterface.GetNumFiles();
}

extern "C" FFMPEGINTERFACE_API void __stdcall EncoderCloseAllFiles()
{
	s_cEncodeInterface.CloseAllFiles();
}

extern "C" FFMPEGINTERFACE_API int __stdcall EncoderGetPicSize(PixelFormat ePixelFmt, 
	int nWidth, int nHeight, int anLinesizes[4])
{
	AVPicture sAVFrameDummy; // av_register_all() is called in the constructor of the s_cEncodeInterface object
	int nSize= avpicture_fill(&sAVFrameDummy, NULL, ePixelFmt, nWidth, nHeight);	// Get size

	if (anLinesizes)	// Only get linesize if wanted
	{
		if (av_image_fill_linesizes(anLinesizes, ePixelFmt, nWidth)<0)	// If it failed set to zero
		{
			memset(anLinesizes, 0, sizeof(anLinesizes[0])*4);
		}
	}
	return nSize;
}

extern "C" FFMPEGINTERFACE_API int __stdcall FFConvertFmts(PixelFormat ePixelFmtIn, 
	int nWidthIn, int nHeightIn, unsigned char* auchBuffer, int nSizeIn, 
	PixelFormat ePixelFmtOut, int nWidthOut, int nHeightOut,
	unsigned char* auchBufOutCompact, int nSizeCompactIn, int* pnSizeCompactOut,
	unsigned char* auchBufOutPadded, int nSizePaddedIn, int* pnSizePaddedOut)
{
	if (!pnSizePaddedOut || pnSizeCompactOut ||						// These must be valid
		(auchBuffer && !auchBufOutCompact && !auchBufOutPadded) ||	// and auchBuffer should also be null if all outputs are NULL
		(!auchBuffer && (auchBufOutCompact || auchBufOutPadded)))	// And auchBuffer should not be null if outputs are not null.
		return BAD_PARAMS;
	
	*pnSizeCompactOut= 0;
	*pnSizePaddedOut= 0;

	// First calculate sizes
	// Calculate the compact size
	/* Switch is required to circumvent bug where avpicture_layout writes by 1024 passed buffer length for these
	   pix fmts. */
	*pnSizeCompactOut= avpicture_get_size(ePixelFmtOut, nWidthOut, nHeightOut);
	switch (ePixelFmtOut) {
    case PIX_FMT_RGB8:
    case PIX_FMT_BGR8:
    case PIX_FMT_RGB4_BYTE:
    case PIX_FMT_BGR4_BYTE:
    case PIX_FMT_GRAY8:
        // Include palette for these pseudo-paletted formats
		(*pnSizeCompactOut)+= 1024;
    }
	// Now the padded size
	AVPicture sAVFrameDummy;
	*pnSizePaddedOut= avpicture_fill(&sAVFrameDummy, NULL, ePixelFmtOut, nWidthOut, nHeightOut);

	if (!auchBuffer) // If we don't need to convert we're done.
		return 0;

	// New we need to convert
	AVFrame* pFrameIn;
	AVFrame* pFrameOut;
	SwsContext*	pConvertCtx;
	bool bError= false;

	// First make sure that the input buffer is large enough to be used in pFrameIn
	if (nSizeIn<avpicture_fill(&sAVFrameDummy, NULL, ePixelFmtIn, nWidthIn, nHeightIn) ||
		auchBufOutCompact?nSizeCompactIn<*pnSizeCompactOut:false ||	// Or if we want to convert to compact form, is buffer too small?
		auchBufOutPadded?nSizePaddedIn<*pnSizePaddedOut:false)		// Or if we want to convert to padded form, is buffer too small?
		return BAD_PARAMS;

	// Conversion context to convert between input/output format
	pConvertCtx= sws_getCachedContext(NULL, nWidthIn, nHeightIn, ePixelFmtIn,
		nWidthOut, nHeightOut, ePixelFmtOut, SWS_BICUBIC, NULL, NULL, NULL);
	// Frame that will store input image
	pFrameIn= avcodec_alloc_frame();
	// Frame that will store output image
	pFrameOut= avcodec_alloc_frame();

	if (!pConvertCtx || !pFrameIn || !pFrameOut)	// Memory issues, or incompatible formats, sizes
	{
		sws_freeContext(pConvertCtx);
		av_free(pFrameIn);
		av_free(pFrameOut);
		return FAILURE;
	}

	// First fill in linesizes for pFrameIn, the buffer will come from auchBuffer
	bError= av_image_fill_linesizes(pFrameIn->linesize, ePixelFmtIn, nWidthIn)<0;
	// Now fill in the padded buffer
	bError = bError || avpicture_fill((AVPicture*)pFrameIn, auchBuffer, ePixelFmtIn, nWidthIn, nHeightIn);

	// Now for the output image, if the padded output buffer is provided we use that, otherwise we allocate the buffer ourselves
	if (auchBufOutPadded)	// We can use this buffer for the output frame
	{
		// First fill in linesizes for pFrameOut
		bError= bError || av_image_fill_linesizes(pFrameOut->linesize, ePixelFmtOut, nWidthOut)<0;
		// Now fill in the padded buffer
		bError = bError || avpicture_fill((AVPicture*)pFrameOut, auchBufOutPadded, ePixelFmtOut, nWidthOut, nHeightOut);
	} else	// We have to create the buffer ourselves
	{
		bError= bError || av_image_alloc(pFrameOut->data, pFrameOut->linesize, nWidthOut, nHeightOut, ePixelFmtOut, 1)<0;
	}

	// First make sure there was no error until now.
	if (bError)
	{
		sws_freeContext(pConvertCtx);
		av_free(pFrameIn);	// Data wasn't allocated by us so don't free it
		if (!auchBufOutPadded)	// Data was allocated by us so we delete it
			av_freep(&pFrameOut->data[0]);
		av_free(pFrameOut);	// Free frame either way
		return FAILURE;
	}

	// Now we are ready to convert
	// If auchBufOutPadded is valid then following function automatically fills in auchBufOutPadded with the data
	// otherwise it gets written to our allocated buffer above.
	bError= sws_scale(pConvertCtx, pFrameIn->data, pFrameIn->linesize, 0, nHeightIn, pFrameOut->data, pFrameOut->linesize)==0;
	// Now if the user also requested the compact representation we return it
	if (auchBufOutCompact)
		bError= bError || avpicture_layout((AVPicture*) pFrameOut, ePixelFmtOut, nWidthOut, nHeightOut, auchBufOutCompact, nSizeCompactIn)<0;

	// Finish up
	sws_freeContext(pConvertCtx);
	av_free(pFrameIn);	// Data wasn't allocated by us so don't free it
	if (!auchBufOutPadded)	// Data was allocated by us so we do delete it
		av_freep(&pFrameOut->data[0]);
	av_free(pFrameOut);	// Free frame either way

	return bError?NO_CONVERT_FMT:0;	
}

extern "C" FFMPEGINTERFACE_API int __stdcall FFCodecPixFmts(CodecID eCodecID, PixelFormat aePixFmts[], int nSize)
{
	AVCodec* pCodec = avcodec_find_encoder(eCodecID);
	if (!pCodec) 
		return -1;
	if (!pCodec->pix_fmts)	// If there's no list, all pix fmts are valid
		return 0;

	int i= 0;
	for (;pCodec->pix_fmts[i] != PIX_FMT_NONE;++i)	// i holds total pix fmts valid, but we copy only nSize
	{
		if (aePixFmts && i<nSize)
			aePixFmts[i]= pCodec->pix_fmts[i];
	}
	return i;
}

extern "C" FFMPEGINTERFACE_API void __stdcall FFPixDesc(PixelFormat ePixFmt, AVPixFmtDescriptor* psPixDesc)
{
	if (psPixDesc)
		*psPixDesc= av_pix_fmt_descriptors[ePixFmt];	// Pix description. This must be valid because we don't check.

	/* The av_pix_fmt_descriptors decleration by default is just extern which MSVC++ cannot use, so add __declspec(dllimport). */
#pragma message("\n<_>_<_>_<_><_>_<_>_<_><_>_<_>_<_><_>_<_>_<_>Remember to add __declspec(dllimport) before the av_pix_fmt_descriptors decleration in\
 pixdesc.h when you update the FFmpeg headers.-<_>_<_>_<_><_>_<_>_<_><_>_<_>_<_><_>_<_>_<_><_>_<_>_<_><_>_<_>_<_><_>_<_>_<_><_>_<_>\n")
}


/////////////////////////////////////////
// Container for all encoding files, implementation
CEncodeInterface::CEncodeInterface()
{
	InitializeCriticalSection(&m_rLock);
	m_nIndex= 0;
	m_cFileList.clear();
	av_register_all();	// So we can access all the encoding ffmpeg funcs.
#ifdef _DEBUG
	if (!s_pfOutFile)
		fopen_s(&s_pfOutFile, "Default FFmpeg DLL Encoder Interface log.txt", "a");
#endif
}

CEncodeInterface::~CEncodeInterface()
{
	CloseAllFiles();	// Make sure all files are closed
	DeleteCriticalSection(&m_rLock);
#ifdef _DEBUG
	fclose(s_pfOutFile);
	s_pfOutFile= NULL;
#endif
}

int CEncodeInterface::OpenFile(const char* szFilename, const SEncodeParams* psEncodeParams, 
	char* szOpt, int* anOptError, int* pnPicSizeInFmt, int anLinesizes[4], int* pnFileID)
{
	if (!pnFileID)
		return BAD_PARAMS;

	CEncodeFile* pcEncodeFile= NULL;
	EnterCriticalSection(&m_rLock);	// Protect access to file list
	pcEncodeFile= new CEncodeFile(m_nIndex);
	m_cFileList.push_back(pcEncodeFile);
	*pnFileID= m_nIndex++;
	LeaveCriticalSection(&m_rLock);
	int nRes;
	if ((nRes= pcEncodeFile->OpenFile(szFilename, psEncodeParams, szOpt, anOptError, pnPicSizeInFmt, anLinesizes))!=0)
	{
		CloseFile(*pnFileID);	// File had trouble so close it
		*pnFileID= -1;
	}
	return nRes;
}

CEncodeFile* CEncodeInterface::GetCEncodeFile(int nFileID)
{
	CEncodeFile* pcEncode= NULL;	// So we return NULL if not found, and so function supports a invalid nFileId
	EnterCriticalSection(&m_rLock);
	for (m_cIterator= m_cFileList.begin();m_cIterator!=m_cFileList.end();++m_cIterator)
	{
		if (nFileID==(*m_cIterator)->m_nIndex)
		{
			pcEncode= *m_cIterator;	// Match
			break;
		}
	}
	LeaveCriticalSection(&m_rLock);
	return pcEncode;
}

int CEncodeInterface::WriteFrame(unsigned char* auchBuffer, int nSizeIn, int nFileID)
{
	CEncodeFile* pcEncode= GetCEncodeFile(nFileID);
	return pcEncode?pcEncode->WriteFrame(auchBuffer, nSizeIn):NO_SUCH_OBJECT;
}

void CEncodeInterface::CloseFile(int nFileID)
{
	CEncodeFile* pcEncode= NULL;	// Assume file doesn't exist, support invalid nFileId.
	EnterCriticalSection(&m_rLock);
	for (m_cIterator= m_cFileList.begin();m_cIterator!=m_cFileList.end();++m_cIterator)
	{
		if (nFileID==(*m_cIterator)->m_nIndex)
		{
			pcEncode= *m_cIterator;
			m_cFileList.erase(m_cIterator);	// Remove from list
			break;
		}
	}
	LeaveCriticalSection(&m_rLock);
	// Delete outside of critical section so as to not hold up everyone.
	delete pcEncode;	// Delete file (and close it), if NULL no effect.
	pcEncode= NULL;
}

void CEncodeInterface::CloseAllFiles()
{
	EnterCriticalSection(&m_rLock);	// Just in case
	for (m_cIterator= m_cFileList.begin();m_cIterator!=m_cFileList.end();)
	{
		delete *m_cIterator;	// Delete object
		m_cIterator= m_cFileList.erase(m_cIterator);	// and remove from list

	}
	LeaveCriticalSection(&m_rLock);
}

int CEncodeInterface::GetNumFiles()
{
	int nRes;
	EnterCriticalSection(&m_rLock);
	nRes= (int) m_cFileList.size();
	LeaveCriticalSection(&m_rLock);
	return nRes;
}




/////////////////////////////////////////////////////////
// Actual encoding file implementation
CEncodeFile::CEncodeFile(int nIndex):m_nIndex(nIndex)
{
	InitializeCriticalSection(&m_rFileSafe);

	m_pFormatCtx= NULL;	// Null everything
	m_pAVStream= NULL;

	m_pAVFrame= NULL;
	m_pAVFrameTmp= NULL;
	m_pConvertCtx= NULL;

	m_auchVideoOutBuf= NULL;
	m_nBuffer= 0;

	m_nPicSizeInFmt= 0;
	m_nFrame= 0;
	m_bDone= false;
}

CEncodeFile::~CEncodeFile()
{
	if (m_pFormatCtx) 
	{
		// In case frames are buffered in the encoder, write the rest of the frames.
		int nFrame= m_nFrame;	// Currently written frames
		while((double(m_pAVStream->pts.val*m_pAVStream->time_base.num) / m_pAVStream->time_base.den<	// Stream pts
			double(nFrame*m_sEncodeParams.nFRDen)/m_sEncodeParams.nFRNom) // is more than the number of frames written, so flush rest
			&& !m_bDone)										   		// No more frames
		{
#ifdef _DEBUG
			fprintf(s_pfOutFile, "File &d, appending end frames\n", m_nIndex);
#endif
			if(WriteFrame(NULL, 0))	// Write remaining frames in buffer
			{
				break;	// Break on error
			}
		}

		/* Write the trailer, if any. */
		av_write_trailer(m_pFormatCtx);
	}
	FinishUp();
	DeleteCriticalSection(&m_rFileSafe);
}

void CEncodeFile::FinishUp()
{
	// If the in and out formats are different we must delete the out frame data buffer
	if (m_pAVFrame) // If this is valid then object data used below (m_pAVStream) is also valid
	{
		if (m_pAVStream->codec->pix_fmt!=m_sEncodeParams.ePixelFmtIn && m_pAVFrame->data[0])
			av_freep(&m_pAVFrame->data[0]);
		av_free(m_pAVFrame);
		m_pAVFrame= NULL;
	}
	// The temp frames data is only temporary data (supplied by user for each pic), so don't delete it.
	if (m_pAVFrameTmp) 
	{
		av_free(m_pAVFrameTmp);
		m_pAVFrameTmp= NULL;
	}
	// Free video memory
	if (m_auchVideoOutBuf) 
	{
		av_free(m_auchVideoOutBuf);
		m_auchVideoOutBuf= NULL;
	}
	m_nBuffer= 0;
	// Close conversion context
	if (m_pConvertCtx)
	{
		sws_freeContext(m_pConvertCtx);
		m_pConvertCtx= NULL;
	}

	// Close video streams
	if (m_pFormatCtx)
	{
		// Close file
		if (m_pFormatCtx->pb && !(m_pFormatCtx->oformat->flags & AVFMT_NOFILE)) 
		{
			avio_close(m_pFormatCtx->pb);
		}

		avformat_free_context(m_pFormatCtx);	// Closes everything
		m_pFormatCtx= NULL;
		m_pAVStream= NULL;
	}
	m_nPicSizeInFmt= 0;
	m_nFrame= 0;
	m_bDone= false;
}

int CEncodeFile::OpenFile(const char* szFilename, const SEncodeParams* psEncodeParams, char* szOpt, int* anOptError, int* pnPicSizeInFmt, 
	int anLinesizes[4])
{
	if (!szFilename || !psEncodeParams || !pnPicSizeInFmt || !anLinesizes)
		return BAD_PARAMS;

	*pnPicSizeInFmt= 0;
	EnterCriticalSection(&m_rFileSafe);

	if (m_pFormatCtx)	// File was already opened
	{
		LeaveCriticalSection(&m_rFileSafe);
		return FILE_OPEN;
	}

	// Save params internally
	m_sEncodeParams= *psEncodeParams;

	/* Auto detect the output format from the filename, otherwise use provided format. */
	avformat_alloc_output_context2(&m_pFormatCtx, NULL, NULL, szFilename);
	if (!m_pFormatCtx) 
	{
		avformat_alloc_output_context2(&m_pFormatCtx, NULL, m_sEncodeParams.szOutFormat, szFilename);
	}
	if (!m_pFormatCtx) 
	{
		LeaveCriticalSection(&m_rFileSafe);
		return E_NO_OUT_FORMAT;
	}

	/* Add the video stream using the default format codecs	and initialize the codecs. */
	if (m_pFormatCtx->oformat->video_codec == CODEC_ID_NONE) // Format doesn't have codec
	{
		FinishUp();
		LeaveCriticalSection(&m_rFileSafe);
		return E_CODEC_NOT_FOUND;
	}
	
	// Add the video stream where we'll save the pictures
	m_pAVStream = av_new_stream(m_pFormatCtx, 0);
	if (!m_pAVStream) 
	{
		FinishUp();
		LeaveCriticalSection(&m_rFileSafe);
		return E_NO_STREAM;
	}
	
	/* Set the codec of our stream.*/
	AVCodecContext* pCodecCtx = m_pAVStream->codec;
	pCodecCtx->codec_id = m_sEncodeParams.eCodecID;
	pCodecCtx->codec_type = AVMEDIA_TYPE_VIDEO;
	/* Save parameters */
	pCodecCtx->bit_rate = m_sEncodeParams.nBitRate;
	/* resolution must be a multiple of two */
	pCodecCtx->width = m_sEncodeParams.nWidth;
	pCodecCtx->height = m_sEncodeParams.nHeight;
	/* Time base: this is the fundamental unit of time (in seconds) in terms
		of which frame timestamps are represented. for fixed-fps content,
		time base should be 1/frame rate and timestamp increments should be
		identically 1. */
	pCodecCtx->time_base.den = m_sEncodeParams.nFRNom;
	pCodecCtx->time_base.num = m_sEncodeParams.nFRDen;
	pCodecCtx->gop_size = 12; /* emit one intra frame every twelve frames at most */

	if (m_sEncodeParams.ePixelFmtOut == PIX_FMT_NONE)	// If out is invalid, use in
		m_sEncodeParams.ePixelFmtOut= m_sEncodeParams.ePixelFmtIn;
	pCodecCtx->pix_fmt = m_sEncodeParams.ePixelFmtOut;
	pCodecCtx->max_b_frames = 1;
	pCodecCtx->thread_count= m_sEncodeParams.nThreadCount;

	if (szOpt && anOptError)	// If the options are specified, parse them
	{
		char* szOptCtx= NULL;
		for (int i= 0;;++i)	// Go until no more options
		{
			char* szOptName= strtok_s(i==0?szOpt:NULL, ",", &szOptCtx);
			char* szOptVal= strtok_s(NULL, ";", &szOptCtx);
			if (!szOptName || !szOptVal)	// Done
				break;
			anOptError[i]= av_set_string3(pCodecCtx, szOptName, szOptVal, 0, NULL);
		}
	}

	// Some formats want stream headers to be separate
	if(m_pFormatCtx->oformat->flags & AVFMT_GLOBALHEADER)
		pCodecCtx->flags |= CODEC_FLAG_GLOBAL_HEADER;

	/* Find the video encoder for this stream's codec. */
	AVCodec* pCodec = avcodec_find_encoder(pCodecCtx->codec_id);
	if (!pCodec) 
	{
		FinishUp();
		LeaveCriticalSection(&m_rFileSafe);
		return E_CODEC_NOT_FOUND;
	}

	// Make sure that the pixel format user selected for this codec is valid
	bool bValid= !pCodec->pix_fmts;	// None specified, so all are valid
	for (int i= 0;!bValid && pCodec->pix_fmts[i]!=PIX_FMT_NONE;++i)
		bValid= pCodec->pix_fmts[i] == m_sEncodeParams.ePixelFmtOut;
	if (!bValid) 
	{
		FinishUp();
		LeaveCriticalSection(&m_rFileSafe);
		return E_INVALID_CODEC_PIXFMT;
	}

	/* Initialize the codecCtx with the codec. */
	if (avcodec_open2(pCodecCtx, pCodec, NULL) < 0) 
	{
		FinishUp();
		LeaveCriticalSection(&m_rFileSafe);
		return E_CODEC_NO_OPEN;
	}

	// Raw picture format requires no copying so no buffer. 
	if (!(m_pFormatCtx->oformat->flags & AVFMT_RAWPICTURE)) 
	{
		// Output buffer for the video file.
		m_nBuffer= max(m_sEncodeParams.nWidth*m_sEncodeParams.nHeight*6+1664, 1024*256);
		m_auchVideoOutBuf = (unsigned char*) av_malloc(m_nBuffer);
		if (!m_auchVideoOutBuf)
		{
			FinishUp();
			LeaveCriticalSection(&m_rFileSafe);
			return ALLOC_MEMORY_ERROR;
		}
	}

	/* Allocate the picture for the encoded pic fmt. If the in/out format are the same, incoming pics 
		are saved in here. Otherwise, it first converted to out format and then saved here. */
	m_pAVFrame =  avcodec_alloc_frame();
	if (!m_pAVFrame) 
	{
		FinishUp();
		LeaveCriticalSection(&m_rFileSafe);
		return ALLOC_MEMORY_ERROR;
	}

	/*	See avpicture_fill, av_image_fill_pointers, avpicture_layout, and avpicture_get_size for how 
		different pixfmts are stored in the linear buffer.
		avpicture_fill will be called on the data buffer of each incoming picture so we don't need to allocate
		a buffer for incoming pics, just the frame. */
	/* if the output format is not the same as the input, then a temporary
		picture with input format is also needed. We also need to provide a conversion context. */
	if (pCodecCtx->pix_fmt != m_sEncodeParams.ePixelFmtIn) 
	{
		/*	Now prepare the buffers that will contain the output image. If the input and output formats are the
			same we don't do this, because we'll fill the input buffer directly with the input images. */
		if (av_image_alloc(m_pAVFrame->data, m_pAVFrame->linesize, pCodecCtx->width, 
			pCodecCtx->height, pCodecCtx->pix_fmt, 1)<0)
		{
			FinishUp();
			LeaveCriticalSection(&m_rFileSafe);
			return ALLOC_MEMORY_ERROR;
		}

		// Frame for input image, no buffer allocation because we use input image directly.
		m_pAVFrameTmp= avcodec_alloc_frame();
		if (!m_pAVFrameTmp) 
		{
			FinishUp();
			LeaveCriticalSection(&m_rFileSafe);
			return ALLOC_MEMORY_ERROR;
		}
		// But we still need to know the linesizes of the input image for the conversion.
		if (av_image_fill_linesizes(m_pAVFrameTmp->linesize, m_sEncodeParams.ePixelFmtIn, pCodecCtx->width)<0)
		{
			FinishUp();
			LeaveCriticalSection(&m_rFileSafe);
			return LINESIZES_ERROR;
		}
		// Get the conversion context
		m_pConvertCtx = sws_getCachedContext(NULL, pCodecCtx->width, pCodecCtx->height, m_sEncodeParams.ePixelFmtIn,
			pCodecCtx->width, pCodecCtx->height, pCodecCtx->pix_fmt, SWS_BICUBIC, NULL, NULL, NULL);
		if (!m_pConvertCtx) 
		{
			FinishUp();
			LeaveCriticalSection(&m_rFileSafe);
			return NO_CONVERT_FMT;
		}

	}
	/* Now open the output file, if needed. */
	if (!(m_pFormatCtx->oformat->flags & AVFMT_NOFILE)) 
	{
		if (avio_open(&m_pFormatCtx->pb, szFilename, AVIO_FLAG_WRITE) < 0) 
		{
			FinishUp();
			LeaveCriticalSection(&m_rFileSafe);
			return E_CANNOT_OPEN_FILE;
		}
	}

	/* Now return the required size of the buffer of the input image. Note, this will include 
		padding and alignment. */
	AVPicture sAVFrameDummy;
	*pnPicSizeInFmt= avpicture_fill(&sAVFrameDummy, NULL, m_sEncodeParams.ePixelFmtIn, 
		pCodecCtx->width, pCodecCtx->height);
	m_nPicSizeInFmt= *pnPicSizeInFmt;	// Save it for later use in copying image
	if (av_image_fill_linesizes(anLinesizes, m_sEncodeParams.ePixelFmtIn, pCodecCtx->width)<0)
	{
		FinishUp();
		LeaveCriticalSection(&m_rFileSafe);
		return LINESIZES_ERROR;
	}

	/* Write the stream header, if any */
	if (avformat_write_header(m_pFormatCtx, NULL)<0)
	{
		FinishUp();
		LeaveCriticalSection(&m_rFileSafe);
		return E_HEADER_NO_WRITE;
	}

	m_nFrame= 0;
	m_bDone= false;
	LeaveCriticalSection(&m_rFileSafe);
	return 0;
}

int CEncodeFile::WriteFrame(unsigned char* auchBuffer, int nSizein)
{

	int nTemp= 0, nRes= 0;
	AVCodecContext* pCodecCtx = m_pAVStream->codec;

	EnterCriticalSection(&m_rFileSafe);

	// Bad object or bad pointers or too small size, when null size must also be zero
	if ((!auchBuffer && nSizein) || (auchBuffer && nSizein<m_nPicSizeInFmt))
		nTemp= BAD_PARAMS;
	if (!m_pFormatCtx)
		nTemp= BAD_OBJECT;
	if (nTemp)
	{
		LeaveCriticalSection(&m_rFileSafe);
		return nTemp;
	}

	// Convert the current image to the proper output format if required
	if (auchBuffer) 
	{
		// Do we need to convert pic into different output format?
		if (pCodecCtx->pix_fmt != m_sEncodeParams.ePixelFmtIn) 
		{
			// Fill in temp pic with input buffer and convert.
			avpicture_fill((AVPicture*)m_pAVFrameTmp, auchBuffer, m_sEncodeParams.ePixelFmtIn,
				pCodecCtx->width, pCodecCtx->height);
			sws_scale(m_pConvertCtx, m_pAVFrameTmp->data, m_pAVFrameTmp->linesize,
				0, pCodecCtx->height, m_pAVFrame->data, m_pAVFrame->linesize);
		} else
		{
			// Fill into output frame directly.
			avpicture_fill((AVPicture*)m_pAVFrame, auchBuffer, m_sEncodeParams.ePixelFmtIn,
				pCodecCtx->width, pCodecCtx->height);
		}
	}
	/* If auchBuffer is NULL then no more frame to compress. The codec has a latency of a few
		frames if using B frames, so we get the last frames by passing NULL frame. */

	// Now save image to video file
	if (m_pFormatCtx->oformat->flags & AVFMT_RAWPICTURE) 
	{
		/* raw video case. The API will change slightly in the near
		future for that. Latency won't happen here.  */
		AVPacket sPkt;
		av_init_packet(&sPkt);

		sPkt.flags |= AV_PKT_FLAG_KEY;
		sPkt.stream_index= m_pAVStream->index;
		sPkt.data= (unsigned char *)m_pAVFrame;
		sPkt.size= sizeof(AVPicture);

		nRes = av_interleaved_write_frame(m_pFormatCtx, &sPkt);
	} else 
	{
		/* Encode the image, if flushing pass null frame. */
		nTemp = avcodec_encode_video(pCodecCtx, m_auchVideoOutBuf, m_nBuffer, 
			auchBuffer?m_pAVFrame:NULL);
		/* If zero size, it means the image was buffered and will need to be flushed later. */
		if (nTemp > 0) 
		{
			AVPacket sPkt;
			av_init_packet(&sPkt);

			// Save pts of frame
			if (pCodecCtx->coded_frame && pCodecCtx->coded_frame->pts != AV_NOPTS_VALUE)
				sPkt.pts= av_rescale_q(pCodecCtx->coded_frame->pts, pCodecCtx->time_base, m_pAVStream->time_base);
			if(sPkt.pts == AV_NOPTS_VALUE && sPkt.dts == AV_NOPTS_VALUE && !(m_pFormatCtx->oformat->flags & AVFMT_NOTIMESTAMPS))
				sPkt.dts= 0;

			// Is it a key frame?
			if(pCodecCtx->coded_frame && pCodecCtx->coded_frame->key_frame)
				sPkt.flags |= AV_PKT_FLAG_KEY;
			sPkt.stream_index= m_pAVStream->index;
			sPkt.data= m_auchVideoOutBuf;
			sPkt.size= nTemp;
			/* Write the compressed frame in the media file */
			nRes = av_interleaved_write_frame(m_pFormatCtx, &sPkt);
		} else 
		{
			nRes = nTemp;	// If zero it's ok, if none zero it means error.
			if (!auchBuffer)	// Either way, no more frames to flush, if flushing
				m_bDone= true;
		}
	}

	if (nRes) 
	{
		LeaveCriticalSection(&m_rFileSafe);
		return E_FRAME_NOT_SAVED;
	}

	m_nFrame++;	// Frame count
	LeaveCriticalSection(&m_rFileSafe);
	return 0;
}