/** 
	Copyright 2011 CPL Lab, Cornell University

    This file is part of ffmpeg interface.

    ffmpeg interface is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    ffmpeg interface is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with ffmpeg interface.  If not, see <http://www.gnu.org/licenses/>.

	The ffmpeg API is copyrighted by their respective copyright holders. See
	the license folder (should have been included with this file) for the 
	copyright notices.
	*/

/** Defines the encoder function accessed by the API. */



#ifndef __ENCODE_INTERFACE_CPL_H__
#define __ENCODE_INTERFACE_CPL_H__

#include <list>
#include "ffi ffmpeg Interface.h"

// Forward declarations
class	CEncodeFile;
struct	AVFormatContext;
struct	AVStream;
struct	AVFrame;
struct	SwsContext;


/** A thread safe class which holds a list of all currently open encoder files and can be used 
	to perform operations on these files. It provides access to these files for the DLL API functions. */
class CEncodeInterface
{
public:
	/** Initializes the instance of this class and registeres the functions required for decoding/encoding. 
		Note, in debug version it opens a FILE object and stores it into a FILE* variable called s_fOutFile. 
		This should have been defined in the implementation file above the class functions. */
					CEncodeInterface();
	/** Also closes the log file. */
	virtual			~CEncodeInterface();

	/** Opens a video file for encoding. @see EncoderOpenFile. 
		Each file that is opened is initialized with a const int value which is used to 
		identify that object. */
	int				OpenFile(const char* szFilename, const SEncodeParams* psEncodeParams, 
						char* szOpt, int* anOptError, int* pnPicSizeInFmt, int anLinesizes[4], int* pnFileID);
	/** Writes the next frame in the video file. @see EncoderWriteFrame */
	int				WriteFrame(unsigned char* auchBuffer, int nSizeIn, int nFileID);
	/** Closes previously opened file. @see EncoderCloseFile. */
	void			CloseFile(int nFileID);
	/** Returns the number of decoding files currently open. @see EncoderGetNumFiles. */
	int				GetNumFiles();
	/** Closes all open video encoding files. @see EncoderCloseAllFiles. */
	void			CloseAllFiles();

private:
	/** Thread safe function that searches the encoder file list for a file with a
		matching index to nFileID and returns the pointer to it if found. Otherwise
		it returns NULL. Pointer returned might become invalid if you call close 
		file for this file ID while you're still using it. */
	CEncodeFile*	GetCEncodeFile(int nFileID);

	/** List of all the open encoding files. */
	std::list<CEncodeFile*>				m_cFileList;
	/** Protects access to the file list. */
	CRITICAL_SECTION					m_rLock;
	/** The index value that will be assigned to the next file that will be opened. 
		It's then incremented. Index value is not reused until overflow. Access is
		also protected by above critical section. */
	int									m_nIndex;
	/** Iterates the file list. */
	std::list<CEncodeFile*>::iterator	m_cIterator;
};


/** A thread safe class which houses a single file that is being encoded. */
class CEncodeFile
{
public:
	/** Initializes the object with a const integer, nIndex that can then be used to locate this object
		in a list. To open the file you need to call {@link OpenFile} on this object.*/
	CEncodeFile(int nIndex);
	/** Also closes the file and finishes writing (if opened). */
	virtual ~CEncodeFile();
	
	/** Opens and associates this object with a particular video file. This creates and initializes all
		the memory and objects needed for decoding. If it was unsuccessful, it'll return an error code, and
		you can try then again to open another video file. However, if it succeeded in opening a video 
		file you won't be able to open another file with this object. @see EncoderOpenFile. */
	int OpenFile(const char* szFilename, const SEncodeParams* psEncodeParams, char* szOpt, int* anOptError, 
		int* pnPicSizeInFmt, int anLinesizes[4]);
	/** Writes the next frame in the video file. @see EncoderWriteFrame 
		Calling with auchBuffer NULL and nSizeIn 0 will force the file buffers to be flushed and any
		internally buffered frames to be written to disk. */
	int	WriteFrame(unsigned char* auchBuffer, int nSizeIn);
	/** A value that gets assigned to this object so that you can identify the object by this value. */
	const int m_nIndex;
private:
		/** Closes all open files, codecs etc. It clears the object to fresh state.
		Is not protected with critical section so calling func must protect it.*/
	void	FinishUp();

	/** Holds all the format info of the file. */
	AVFormatContext*	m_pFormatCtx;
	/** Holds the pointer to the stream to which we're adding frames. */
	AVStream*			m_pAVStream;
	/** Holds the parameters that was used to open this file. {@link OpenFile}*/
	SEncodeParams		m_sEncodeParams;

	/** The buffer that is used when writing the encoded frames to the disk.*/
	unsigned char*	m_auchVideoOutBuf;
	/** The size of the above buffer. */
	int				m_nBuffer;

	/** The frame in which the final image to be written to disk is held. */
	AVFrame*	m_pAVFrame;
	/** If the image format passed by user is different than the image format that is written to disk, this 
		frame holds the image passed by user, otherwise it's NULL. */
	AVFrame*	m_pAVFrameTmp;
	/** If the image format passed by user is different than the image format that is written to disk, this 
		holds the context used to convert between the different formats, otherwise it's NULL. */
	SwsContext*	m_pConvertCtx;
	/** Holds the size of the image that the user will pass in. nSizeIn in WriteFrame() should be at least this
		large. */
	int			m_nPicSizeInFmt;
	/** Holds the count of the number of frames currently passed to be written. */
	int			m_nFrame;
	/** After the user finished writing frames we might have to flush our buffers and force it to write any remaining
		frames that were buffered. When there are no more frames to be flushed this is set to true indicating that
		we're done. */
	bool		m_bDone;

	/** Protects access to this object. */
	CRITICAL_SECTION m_rFileSafe;
};

#endif