/** 
	Copyright 2011 CPL Lab, Cornell University

    This file is part of ffmpeg interface.

    ffmpeg interface is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    ffmpeg interface is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with ffmpeg interface.  If not, see <http://www.gnu.org/licenses/>.

	The ffmpeg API is copyrighted by their respective copyright holders. See
	the license folder (should have been included with this file) for the 
	copyright notices.
	*/


/** Defines the DLL interface to ffmpeg. */


#ifndef __FFMPEG_INTERFACE_CPL_H__
#define __FFMPEG_INTERFACE_CPL_H__


extern "C" {
	/** Imports the ffmpeg API constants. */
#include "libavutil\pixfmt.h"	/** Defines the PixelFormats. */
#include "libavcodec\avcodec.h"	/** Defines the CodecIDs. */
#include "libavutil\pixdesc.h"	/** Defines the pixel formats description. */
}


// The following ifdef block is the standard way of creating macros which make exporting 
// from a DLL simpler. All files within this DLL are compiled with the FFMPEGINTERFACE_EXPORTS
// symbol defined on the command line. This symbol should not be defined on any project
// that uses this DLL. This way any other project whose source files include this file see 
// FFMPEGINTERFACE_API functions as being imported from a DLL, whereas this DLL sees symbols
// defined with this macro as being exported.
#ifdef FFMPEGINTERFACE_EXPORTS
#define FFMPEGINTERFACE_API __declspec(dllexport)
#else
#define FFMPEGINTERFACE_API __declspec(dllimport)
#endif




/** Decoder/encoder error return flags. */
/** You tried to open a file that was already opened. This error never occurs to the user. */
#define	FILE_OPEN			1
/** A decoder error where you tried to open a video file but it couldn't be opened. */
#define D_CANT_OPEN_FILE	2
/** Decoder error where we couldn't read the video file stream. */
#define D_CANT_READ_FILE	3
/** Decoer error where the decoder required to decode the file couldn't be found. */
#define D_CANT_FIND_DECODER	4
/** Decoer error where the decoder required to decode the file couldn't be opened. */
#define D_CANT_OPEN_DECODER	5
/** A decoder error where the pixel format selected to return frames to the user in is invalid. */
#define	D_NO_FMT			6
/** A general error that memory couldn't be allocated for an object. */
#define	ALLOC_MEMORY_ERROR	7
/** General error where we couldn't convert a picture in one pixel format to the target pixel format. */
#define	NO_CONVERT_FMT		8
/** A general error where you tried to operate on a bad internal object. This error is never returned to the user. */
#define	BAD_OBJECT			9
/** Decoder error where the decoder failed to seek. */
#define	D_SEEK_FAILED		10
/** A general error where you tried to operate on an invalid file. Typically occurs when you pass in an invalid fileID. */
#define	NO_SUCH_OBJECT		11
/** A general error where the parameters you passed in to a function call was bad. Examples are NULL pointers where none
	are allowed, the size of a buffer is smaller than required etc. */
#define	BAD_PARAMS			12
/** A decoder warning, it simply indicates that we decoded past the last frame and the picture returned is actually the
	first frame of the video file since we looped back to the start. */
#define	D_LAST_FRAME		13
/** An encoder error where we couldn't find the proper format in which to write the video file. Proper formats are avi, wmv etc.
	if we couldn't figure out the type of video file you wish to create this is returned. */
#define	E_NO_OUT_FORMAT		14
/** An encoder error indicating that the last frame you asked to write wasn't written due to encoding or writing issues. */
#define	E_FRAME_NOT_SAVED	15
/** An encoder warning indicating that we couldn't create the file on disk. */
#define	E_CANNOT_OPEN_FILE	16
/** An encoder error indicating we couldn't calculate linsizes for this particular format. */
#define	LINESIZES_ERROR		17
/** An encoder error indicating that we couldn't add a video stream to the video file. */
#define	E_NO_STREAM			18
/** An encoder error indicating we couldn't find an codec with which to encode the video frames. */
#define	E_CODEC_NOT_FOUND	19
/** An encoder error indicating that we couldn't open the codec required to encode the video frames. */
#define	E_CODEC_NO_OPEN		20
/** A decoder warning indicating that we failed to seek to the required position. However, it's different than D_SEEK_FAILED
	because this indicates that we ended up seeking to the start of the video file. */
#define	D_SEEK_RESET		21
/** An encoder error indicating that we couldn't write the header for the video file. */
#define	E_HEADER_NO_WRITE	22
/** A general error indicating some failure. Might occur because we don't care about the exact error so it indicates that
	one among many error occurred. */
#define	FAILURE				23
/** An encoder error indicating that the pixel format you selected as the output pixel format for the video is
	not allowed for the video codec you're using. You can get the allowed format for each codec with FFCodecPixFmts(). */
#define E_INVALID_CODEC_PIXFMT	24




/** Decoder API. 
An ffmpeg based decoder that decodes a video file into individual frames.
In order to read a file you call DecoderOpenFile to open a video file and if successful
you can then ask to get subsequent frames or seek in the video file. Frames are returned
in a particular pixel format. You can select which format the output picture will be, or
leave it in the same format as the video file. For example, if the format of the pictures 
in the video file is RGB24 you can ask to return individual frames in RGB24, gray8 or any 
other pixel format. Possible pixel formats are defined in the PixelFormat enum in pixfmt.h
in the ffmpeg API. Of course, it might not be possible to convert a particular format 
into another, in which case the open function returns an error. If you don't know about
pixel formats you should select to return the picture in one of the RGB formats because
those are linear and relatively simple.

After being done with the file you should close it to release the resources. However, 
all files are automatically closed when the dll unloads. Note, all functions are thread 
safe, except the close file function, which as expected cannot be called as long as any 
other function is operating on the file that you wish to close.

When opening a file, some information about the file is returned so that you know how
to display the file to the user. This info is returned in the SDecodeFileParams struct.

In debug versions of the code, a log file is created and some file info is saved there. The
file name is "Default FFmpeg DLL Decoder Interface log.txt". */

/** Holds info about a particular video file. Some parameters might not always be available
	and are set to zero in that case. */
typedef struct SDecodeFileParams
{
	/** The estimated duration of the video file, in seconds. Might be zero if unknown. */
	double		dDuration;
	/** A number identifying this file, the number isn't reused when the file is closed until overflow.
		-1 if unsuccessful. When opening a file you get assigned this number and in subsequent calls to the decoder
		you pass in this number to target that file. 
		The decoder and encoder file IDs are kept separately so you could have the same IDs referring to
		different files in the encoder and decoder.*/
	int			nFileId;
	/** The picture format that will be used to output the picture in DecoderGetNextFrame calls.
		This is the format of the returned picture. Naturally the picture structure in the returned buffer
		is different for different pixel formats. */
	PixelFormat	ePixFmtOut;
	/** The width of the pictures in the file. */
	int			nWidth;
	/** The height of the pictures in the file. */
	int			nHeight;
	/** The size of the buffer that is passed into the DecoderGetNextFrame function, and the size of the
		returned picture in bytes. For example, for a RGB24 pixel format with frame width and height of 640x480 
		where each pixel is 3 bytes deep the buffer size will be 3*640*480 bytes. This size and representation
		of the pixel format is determined by ffmpeg and by the particular pixel format. */
	int			nBufferSize;
	/** The estimated number of frames in the file. Might be zero if unknown. */
	int			nFrames;
	/** The nominator of the frame rate. Might be zero if unknown. */
	int			nMeanFrameRateNom;
	/** The denominator of the frame rate. Might be zero if unknown. */
	int			nMeanFrameRateDen;
	/** Some pixel formats have multiple planes, this param returns the number of bytes in each
		line of each plane for the ePixFmtOut pixel format. For example, for RGB24 anLinesizes[0] is 3*nWidth
		and all other planes are zero. For YUV420P, there are three planes defined and only anLinesizes[3] 
		is empty. For pal8 for example, anLinesizes[0] is the actual data while anLinesizes[1] is the palette.
		These values are defined in ffmpeg for each format. */
	int			anLinesizes[4];
}SDecodeFileParams;


/** Opens a single file for decoding. The function is thread safe.

@param szFilename null terminated string containing the filename of the file you want to open for decoding. Should include the full path.
@param aePixFmtPreferred	once the file is opened successfully you can start requesting frames with {@link DecoderGetNextFrame}. That 
	function returns the frames in a particular pixel format. This parameter is an array of pixel formats and controls the format in which 
	the frame will be returned. If this parameter is NULL, the format will be the same as that of the video file. If the length of this array
	is one, aePixFmtPreferred[0] is the format that will be used. Finally, if the length is larger than one we will chose the best pixel format 
	from this array of pixel formats. We choose pixel formats so that the least information is lost when converting. For example, if the 
	video file pixel format is RGB24 and aePixFmtPreferred is {PIX_FMT_RGB16, PIX_FMT_GRAY8} we will choose PIX_FMT_RGB16 as the output
	pixel format so as to not lose color. The formula used to determine the best format is avcodec_find_best_pix_fmt().
	Note, the pixel format chosen for output will be returned in the psDecodeFileParams parameter.
	Also note, when finding the best pixel formats we assume that the video file doesn't make use of alpha. That is even if the video pixel format
	is RGBA, we assume that the actual images do not use alpha.

	If you're unsure if we support certain format conversions 
	or file types you could run a test video to see if errors are returned. Alternatively, because
	we support everything that ffmpeg supports you could use the ffmpeg, ffplay, and ffprobe exes
	of the same version as this code (which should be included with this code) to find out what
	formats we can write in. See the ffmpeg documentation on the use of those ff binaries.

@param nFmts the length of the aePixFmtPreferred array. Should be zero if aePixFmtPreferred is NULL.
@param psDecodeFileParams struct that returns info about the file opened, if successful. Note, Some parameters might 
	not always be available. 
@return	one of the error codes if the file couldn't be opened properly (and no file was therefore opened), zero otherwise. 
	On error, nFileId in psDecodeFileParams is -1. */
extern "C" FFMPEGINTERFACE_API int __stdcall DecoderOpenFile(const char szFilename[], PixelFormat aePixFmtPreferred[], 
	int nFmts,  SDecodeFileParams* psDecodeFileParams);
/** Once a file is opened, you can request the next frame in this video. When calling this right after you opened 
	the file, the video will start at the first frame. However, after you call the seek function {@link DecoderSeekToTimestamp},
	this function will return the first frame at the position you seeked to. Following this, the function simply returns the next
	frame following the previous frame. 
	When the last frame is reached, the last frame is simply returned. However, upon a subsequent call to this function the position
	will loop back to the start of the video, the first frame will be returned and the function returns the D_LAST_FRAME code.
	This way the video will be played in a loop.
	The function is thread safe relative the all the functions (and itself) except relative to the close file functions.
	So the close functions should not be called while this function is running. 

@param nFileId the index targeting the file you want the next frame from. This should be the nFileId value you received 
	in the psDecodeFileParams struct when opening the file. An invalid file number will simply return an error.
@param auchBuffer a buffer into which the frame will be copied into. The format of the picture returned in the buffer is
	the same as the ePixFmtOut PIX_FMT_ variable in the psDecodeFileParams struct in {@link DecoderOpenFile}. For example, for RGB24
	it'd simply be RGBRGBRGB... 
	Note, the avpicture_layout function from the ffmpeg API will be used to copy the image into the buffer. This results in a
	more compact representation of the picture. You can study that function to find out how the picture is represented in the buffer.
@param nSizeIn the size of the buffer, auchBuffer in bytes. This should be at least as large as the nBufferSize variable in the 
	psDecodeFileParams struct returned in {@link DecoderOpenFile}. If it's less an error will be returned because the picture won't fit 
	in the buffer.
@param pdPts returns the time (in seconds) at which the picture returned should be displayed to the user. This value is probably more 
	accurate than calculating picture number divided by average frame rate, if you were to do this yourself by counting frames returned. 
	This time is in video time (i.e. relative to the start of the video). 
@return one of the error codes if something went wrong or D_LAST_FRAME if the last frame of the video file was returned at the previous call
	and the frame currently returned is the first frame of the video because we play it in a loop and we looped back. */
extern "C" FFMPEGINTERFACE_API int __stdcall DecoderGetNextFrame(int nFileId, unsigned char auchBuffer[], 
	int nSizeIn, double* pdPts);
/** Seeks in the video to the desired time in video time. Following this call, calls to DecoderGetNextFrame returns 
	frames starting at the seeked position.
	Note, in some videos (such as compressed h264 with B frmaes) you cannot seek to every frame in the video file, 
	therefore, the closest frame to dTimestamp that we "can" seek to will be seeked to.
	The function is thread safe relative the all the functions (and itself) except relative to the close file functions.
	So the close functions should not be called while this function is running. 

@param nFileId the index targeting the file you want to seek in. This should be the nFileId value you received 
	in the psDecodeFileParams struct when opening the file. An invalid file number will simply return an error.
@param dTimestamp the time (in seconds) in video time that you want to seek to.
@return one of the error codes if something went wrong, zero otherwise. */
extern "C" FFMPEGINTERFACE_API int __stdcall DecoderSeekToTimestamp(int nFileId, double dTimestamp);
/** Closes a file you previously opened with {@link DecoderOpenFile}. 
	Function is thread safe with itself but not with other functions. So don't call it while any other function is running.
	After closing the nFileId value won't be reused (until overflow).

@param nFileId the index targeting the file you want to close. This should be the nFileId value you received 
	in the psDecodeFileParams struct when opening the file. Using an invalid file number will simply do nothing.
@return void.*/
extern "C" FFMPEGINTERFACE_API void __stdcall DecoderCloseFile(int nFileId);
/** Returns the number of files currently open for decoding. This function is completely thread safe. 

@return the number of decoding files currently open. */
extern "C" FFMPEGINTERFACE_API int __stdcall DecoderGetNumFiles();
/** Closes all currently open decoding files. Function is thread safe with itself but not with 
	other functions. So don't call it while any other function is running.

@return void.*/
extern "C" FFMPEGINTERFACE_API void __stdcall DecoderCloseAllFiles();
/** Returns the pixel format of the video file. You can use this to determine the pixel format that the pictures are stored in
	in the video file.

@param szFilename null terminated string containing the filename of the file you want to target. Should include the full path.
@return the pixel format of the video file in question. -1 in case of error. */
extern "C" FFMPEGINTERFACE_API int __stdcall DecoderPeekPicFmt(const char szFilename[]);



/** Encoder API 
An ffmpeg based encoder which lets you create a video file and write frames to it.

You create a new video file by calling EncoderOpenFile with suitable parameters. These parameters define 
the format of the video file as well as the format of the frames you'll pass in later. After the video
file has been successfully opened you can start to write frames to the video file with EncoderWriteFrame. 
The frames are written at identical intervals of 1/frame rate (ms). The format of the picture that 
you pass in is determined by the parameters you selected when opening the file. You have to make sure 
that the pictures you pass in match that format. 

After being done with the file you should close it to release the resources and finish writing the buffers. 
However, all files are automatically closed when the dll unloads. Note, all functions are thread safe, 
except the close file function, which as expected cannot be called as long as any other function is operating
on the file that you wish to close.

In debug versions of the code, a log file is created and some file info is saved there. The
file name is "Default FFmpeg DLL Encoder Interface log.txt". */


/** A struct used to initialize a file when you open a new file for encoding. 
	There are four important parameters you have to supply when creating a video file: 
	The file container in which the video file will be saved to (avi, mp4, wmv etc.) 
	this is determined from the filename and {@link szOutFormat} if the filename is unclear.
	The codec with which the program will write the pictures to the video file. Examples 
	are h264, MPEG4, and rawvideo (no compression). This is determined by the {@link eCodecID}
	parameter and by the list of codecs defined in avcodec.h.
	The pictures pixel format in which the picture will be saved as and in which the user
	will send the picture to us. If they are different we will convert between the formats.
	Example of both are rgb24, rgb16, gray8, pal8 (not supported as output only input). All
	these are defined in pixfmt.h. If you're unsure if we support certain format conversions 
	or outputs you could run a test video to see if errors are returned. Alternatively, because
	we support everything that ffmpeg supports you could use the ffmpeg, ffplay, and ffprobe exes
	of the same version as this code (which should be included with this code) to find out what
	formats we can write in. See the ffmpeg documentation on the use of those ff binaries.
	.*/
typedef struct SEncodeParams
{
	/** The codec with which to write the video file. This could
		be any of the ffmpeg defined codecs. Example codecs are CODEC_ID_RAWVIDEO
		for writing uncompressed video files or CODEC_ID_MPEG4 for writing h264 compressed
		files. CODEC_ID_RAWVIDEO works well, any other compression codec is supported but
		we don't provide many parameters for those codecs. All the parameters used to set
		how much the codec will compress the video are set to defaults. The only explicit 
		parameter provided is nBitRate. Therefore, if you're using any codec that requires
		fine tuning of other parameters you're limited to defaults. Note, CODEC_ID_RAWVIDEO 
		doesn't require any compression parameters since it doesn't compress the video.*/
	enum CodecID	eCodecID;
	/** If you're using any compression codec you can set the bit rate for the video file
		with this parameter. This is ignored for CODEC_ID_RAWVIDEO. */
	int				nBitRate;
	/** The width of the pictures that will be written to the video file. Naturally, all
		the pictures added to the file must have the same width. Needs to be an even number.*/
	int				nWidth;
	/** The height of the pictures that will be written to the video file. Naturally, all
		the pictures added to the file must have the same height. Needs to be an even number.*/
	int				nHeight;
	/** The denominator of the frame rate of the video file. Only constant frame rates 
		are supported. When adding frames to the video file you won't provide a time stamp
		of the frame because it's assumed that each frame is spaced by 1/frame rate (ms). 
		Just add duplicate frames if you want different spacing. However, it has to be 
		multiples of 1/frame rate (ms). For a frame rate of 29.97 frames/sec for example, 
		this will be 100. */
	int				nFRDen;
	/** The nominator of the frame rate of the video file. For a frame rate of 29.97 
		frames/sec for example, this will be 2997. */
	int				nFRNom;
	/**	Selects the format of the pictures that the user will send using {@link EncoderWriteFrame}. 
		In that function the user inputs a picture in a buffer that will be written to the video file.
		This parameter describes the structure of the picture in that buffer. */
	enum PixelFormat ePixelFmtIn;
	/** Selects the pixel format in which the pictures will be saved to disk. You input a picture 
		to {@link EncoderWriteFrame} with structure ePixelFmtIn, however, you might want us to save
		the picture to the file in a different format. This parameter selects the format in which it'll
		actually be written. Naturally if ePixelFmtIn is different from ePixelFmtOut we will convert
		the image internally to ePixelFmtOut before saving. For example, if ePixelFmtIn is PIX_FMT_RGB24
		and ePixelFmtOut is PIX_FMT_GRAY8 we'd naturally convert the frame to PIX_FMT_GRAY8 before writing
		to disk. 
		If it's PIX_FMT_NONE (-1) then ePixelFmtIn will be used. */
	enum PixelFormat ePixelFmtOut;
	/** Selects how many threads will be used by the encoder to encode the videos. If there're multiples CPUs
		availible this will make it use this many CPUs. if there're more threads then CPUs availible,
		it will emulate having mutliple threads. Setting this to zero will make it automatically select
		the optimum number of threads. However, not all codecs support zero or allow more than one thread
		so the program might not work if set to zero. You can find by trial and error what works.
		For libx264 (H264) for example zero will work and will make it use all CPUs. */
	int				nThreadCount;
	/** Sets the file type of the video file that we want to save. First we look at the filename of the file
		that you want to write to see if we can determine the container file type of the video file that
		will be saved. If we cannot we will use this null terminated string parameter. For example, it
		might be avi in case the file will be save as an avi video file. This should be set in addition to 
		the filename. */
	char			szOutFormat[12];

	SEncodeParams()
	{
		memset(&eCodecID, 0, sizeof(SEncodeParams));
	}
}SEncodeParams;

/** Creates a video file into which we'll save picture frames. The function is thread safe. 

@param szFilename null terminated string containing the filename of the file you want to create. Should include the full path.
@param psEncodeParams a struct that you pass in containing parameters describing the video file that you want to create. See 
			{@link SEncodeParams} for a full description of these parameters. You need to make sure the parameters are valid,
			otherwise the function will return an error.
@param szOpt a null terminated string containing additional options for the encoder. The options are identical to the options
			listed in the FFmpeg executable documentation or their help page. The format of the options in the string is
			the option name folowed by a comma followed by the option value followed by a semicolon. For example: 
			"qmin,12;qmax,40;". If this format isn't followed or the option isn't found an error will be returned for the 
			option that failed. You can experiment to find which options are allowed. You can even use preset files if
			the files can be found. If this or anOptError is NULL this will be ignored.
@param anOptError an array of at least length as long as the number of options in szOpt and upon return, each index will
			contain an FFmpeg error code corresponding to the option at that index, if an error occured at this option.
			You can use this to find how to properly supply the options. In the example above, this should be of length 2.
@param pnPicSizeInFmt returns the size of the picture that you'll pass in using the {@link EncoderWriteFrame} function. 
			The nSizeIn parameter in that function must be at least as large as the value returned here. We should note
			that the picture size returned here is possibly larger than the picture size returned when opening a decoder file
			of the same format because we require the pictures in the encoder to be padded and aligned. So this representation
			is not compact as opposed to the decoder representation. {@see EncoderGetPicSize}
@param anLinesizes returns the linsizes of the images that you'll pass into the {@link EncoderWriteFrame} function. 
			These values should help you make sure that the image you're passing in is of the same structure as we expect it to be
			according to the pixel format that you specified in psEncodeParams.
			Some pixel formats have multiple planes, this param returns the number of bytes in each	line of each plane for that
			pixel format. For example, for RGB24 anLinesizes[0] is 3*nWidth	and all other planes are zero. For YUV420P, there 
			are three planes defined and only anLinesizes[3] is empty. For pal8 for example, anLinesizes[0] is the actual data 
			while anLinesizes[1] is the palette. You have to make sure that the image you pass in will match the expected structure
			of that image. These values are defined in ffmpeg for each format. {@see EncoderGetPicSize}
@param pnFileID A number identifying this file. The number isn't reused when the file is closed until overflow.
			-1 if unsuccessful. When opening a file you get assigned this number and in subsequent calls to the encoder
			you pass in this number to target that file. The decoder and encoder file IDs are kept separately so you could have 
			the same IDs referring to different files in the encoder and decoder.
@return	one of the error codes if the file couldn't be opened properly (and no file was therefore opened), zero otherwise. 
			On error, pnFileID is -1. */
extern "C" FFMPEGINTERFACE_API int __stdcall EncoderOpenFile(const char* szFilename, const SEncodeParams* psEncodeParams, 
	char* szOpt, int* anOptError, int* pnPicSizeInFmt, int anLinesizes[4], int* pnFileID);
/** Writes a single frame to the previously opened video file. This function appends a single frame at the end of the video file
	with a time difference of 1/frame rate as defined in psEncodeParams since the last frame. 
	The function is thread safe relative the all the functions (and itself) except relative to the close file functions.
	So the close functions should not be called while this function is running. 

@param auchBuffer a buffer containing the frame to be written. The structure of the frame must match the structure of ePixelFmtIn
			as defined in psEncodeParams. You must know the structure expected for each type of pixel format that you use. You can
			use {@link EncoderGetPicSize} to find out more about a format. Note that this structure is not the same as the decoding 
			structure because this structure requires padding and alignment. You might need to experiment to find how the function
			expects the image to appear. For example, if using RGB24 with a picture width and height of 640x480 then the compact 
			representation in the decoder would simply be an image of size 3*640*480 with the format of RGBRGBRGB... But in this 
			representation with padding it might, for example require a picture of size 3*640*480+24 as input. In this case you simply add
			24 bytes after your frame with empty data so that the buffer is large enough. For planer data however (such as a YUV format),
			you'd have to know how the different planes are expected to be stored in this linear buffer. 
			Note, the avpicture_fill function is used to write the buffer into a proper picture that can be saved to disk. Study that function
			to find out more about how the structure of the picture is expected to be in the buffer. Also, FFConvertFmts shows the
			expected picture structure for the padded form.
@param nSizeIn the size of auchBuffer. This must be at least as large as the value returned in pnPicSizeInFmt in {@link EncoderOpenFile}.
@param nFileID the index targeting the file for which you want to write the next frame. This should be the pnFileID value you 
			received when opening the file. An invalid file number will simply return an error.
@return one of the error codes in case of error, zero otherwise. */
extern "C" FFMPEGINTERFACE_API int __stdcall EncoderWriteFrame(unsigned char* auchBuffer, int nSizeIn, int nFileID);
/** Closes a file you previously opened with {@link EncoderOpenFile}. 
	Function is thread safe with itself but not with other functions. So don't call it while any other function is running.
	After closing the nFileID value won't be reused (until overflow).
	Note that until you call this function on a open file, the video file would not be completely written.

@param nFileID the index targeting the file you want to close. This should be the pnFileID value you received 
	when opening the file. Using an invalid file number will simply do nothing.
@return void.*/
extern "C" FFMPEGINTERFACE_API void __stdcall EncoderCloseFile(int nFileID);
/** Returns the number of files currently open for encoding. This function is completely thread safe. 

@return the number of encoding files currently open. */
extern "C" FFMPEGINTERFACE_API int __stdcall EncoderGetNumFiles();
/** Closes all currently open encoding files. Function is thread safe with itself but not with 
	other functions. So don't call it while any other function is running.

@return void.*/
extern "C" FFMPEGINTERFACE_API void __stdcall EncoderCloseAllFiles();
/** Returns the linesizes and size in bytes of a picture of any pixel format including padding and alignment. You can use this to find
	out what kind of structure a particular pixel format requires. The returned values are only true for encoding files, for 
	decoding files the size might be smaller since its representation is more compact. 

@param ePixelFmt the pixel format for which you want to find its structure. Supported pixel formats are defined in pixfmt.h.
@param nWidth the width of the sample image.
@param nHeight the height of the sample image.
@param anLinesizes the linsizes of this sample image. Some pixel formats have multiple planes, this param returns the number of bytes in each
			line of each plane for that	pixel format. For example, for RGB24 anLinesizes[0] is 3*nWidth	and all other planes are zero. 
			For YUV420P, there are three planes defined and only anLinesizes[3] is empty. For pal8 for example, anLinesizes[0] is the actual data 
			while anLinesizes[1] is the palette. These values are defined in ffmpeg for each format.
@return the expected size of a complete image of this format. */
extern "C" FFMPEGINTERFACE_API int __stdcall EncoderGetPicSize(PixelFormat ePixelFmt, 
	int nWidth, int nHeight, int anLinesizes[4]);
/** Converts an image from one pixel format to another. The input image buffer is of pixel format ePixelFmtIn
	and must be padded while we offer two output pictures, both of pixel format ePixelFmtOut. One output picture
	is copied into the auchBufOutPadded buffer in padded form while the other is copied into the auchBufOutCompact
	buffer in compact form. In addition, if all the buffers are NULL we return the required size of the (output) frames
	
	ffmpeg can output pictures in two formats, one is a compact form while the other is padded and aligned. The compact
	representation is simple, however, the padded form might not be. This function in addition to converting a picture
	from one format to another will also return the output in both forms which should help you see the differences
	between the padded and compact form for any pixel format. For example, if you start with RGB24 the pixels will be 
	packed as RGBRGBRGB... with any padding added to the end as zeros. Now you can convert it to a planer format such as
	PIX_FMT_YUV420P and see how that format looks in the linear buffer in both formats. Typically when you input a image
	to the ffmpeg API you'd have to supply the image in padded form, so this should help understanding what ffmpeg 
	expects for these pictures. Note, the relevant ffmpeg functions for padded and compact picture copying is 
	avpicture_fill() and avpicture_layout(), respectively. Also note, the decoder returns images in compact form while the 
	encoder expects to receive images in padded form.
	
	You can also convert the image to the same pixel format as the original image. It'd simply copy the image to the 
	output buffers. Also, you can rescale the image any different widths and heights. 
	
@param ePixelFmtIn the pixel format of the input image in auchBuffer. Possible ffmpeg pixel formats are defined in 
			pixfmt.h. 
@param nWidthIn the width of the input image frame. 
@param nHeightIn the height of the input image frame. 
@param auchBuffer the buffer containing the actual frame. The structure of the data needs to match ePixelFmtIn and 
			needs to be in the ffmpeg padded format. See below for when this can be NULL.
@param nSizeIn the size of the auchBuffer buffer. This includes padding. 
@param ePixelFmtOut the pixel format of of the output image. If this is different than ePixelFmtIn the output picture will
			be converted to ePixelFmtOut. Otherwise, the picture is simply copied (and rescaled). 
@param nWidthOut the width of the output image. This can be different than nWidthIn in which case the image is rescaled to 
			the desired width. 
@param nHeightOut the height of the output image. This can be different than nHeightIn in which case the image is rescaled to 
			the desired height. 
@param auchBufOutCompact the buffer that will contain the output picture in compact form. If this is NULL it will be ignored.
			However, even if this is NULL, pnSizeCompactOut will still return the size of the compact representation of a picture of
			output height and width of output pixel format. Calling with one output buffer set to NULL will still convert the
			picture to the output format of the other output buffer if that's not NULL. If both output buffers are NULL then 
			the input buffer must also be NULL otherwise an error is returned. Also, if the input buffer is NULL, both output
			buffers must be NULL otherwise an error is returned.
@param nSizeCompactIn the size of auchBufOutCompact. This should be at least the required size for the compact representation 
			of the output pixel format picture otherwise a error is returned. 
@param pnSizeCompactOut returns the required size of a picture of format ePixelFmtOut and width and height of 
			nWidthOut, nHeightOut in compact form. You can call the function with all the buffers set to NULL to return
			this zise without having to convert a picture. Returns an error if this parameter is NULL. 
@param auchBufOutPadded the buffer that will contain the output picture in padded form. If this is NULL it will be ignored.
			However, even if this is NULL, pnSizePaddedOut will still return the size of the padded representation of a picture of
			output height and width of output pixel format. Calling with one output buffer set to NULL will still convert the
			picture to the output format of the other output buffer if that's not NULL. If both output buffers are NULL then 
			the input buffer must also be NULL otherwise an error is returned. Also, if the input buffer is NULL, both output
			buffers must be NULL otherwise an error is returned.
@param nSizePaddedIn the size of auchBufOutPadded. This should be at least the required size for the padded representation 
			of the output pixel format picture otherwise a error is returned. 
@param pnSizePaddedOut returns the required size of a picture of format ePixelFmtOut and width and height of 
			nWidthOut, nHeightOut in padded form. You can call the function with all the buffers set to NULL to return
			this size without having to convert a picture. Returns an error if this parameter is NULL. 
@return returns an error code in case of error, zero otherwise. An error will be returned if either pnSizeXXXOut is NULL
			or if the input buffer is NULL while any of the output buffers aren't or if the input buffer isn't NULL but
			both output buffers are. A general failure error or NO_CONVERT_FMT might be returned if something went wrong 
			with the conversion. */
extern "C" FFMPEGINTERFACE_API int __stdcall FFConvertFmts(PixelFormat ePixelFmtIn, 
	int nWidthIn, int nHeightIn, unsigned char* auchBuffer, int nSizeIn, 
	PixelFormat ePixelFmtOut, int nWidthOut, int nHeightOut,
	unsigned char* auchBufOutCompact, int nSizeCompactIn, int* pnSizeCompactOut,
	unsigned char* auchBufOutPadded, int nSizePaddedIn, int* pnSizePaddedOut);

/** Returns the valid pixel formats for each codec. Each codec can write video files in different pixel formats. 
	This function tells you which pixel formats are supported by the particular codec. 
	
@param eCodecID the codec that will be used to write the video file. 
@param aePixFmts an array of length nSize into which if not NULL we'll copy the valid pixel formats. The number of 
			formats copied is the minimum of nSize and the return value of the function. If it's NULL none are copied.
@param nSize the length of the aePixFmts array. 
@return the number of valid pixel formats for this codec. If -1 it means  there's an error in the parameters, i.e. the
			codec is invalid. 0 means all the formats are supported and any other number simply means that's how many 
			format are supported. You can call the function once with the array params NULL so it'll return the 
			required length of the array then subsequently calling it with nSize set to that length so that it'll
			copy all the valid formats.*/
extern "C" FFMPEGINTERFACE_API int __stdcall FFCodecPixFmts(CodecID eCodecID, PixelFormat aePixFmts[], int nSize);
/** Returns the pixel format description for a particular pixel format. This descriptor is described in the FFmpeg
	pixdesc.h header. The descriptor describes that exact format of the data for this pixelf format in their
	image buffers.

	Warning, ePixFmt has to be a valid pixel format because it's not checked for validity. Using an invalid
	pixel format might crash the program. In particular PIX_FMT_NONE is not allowed. Also, if you substitue
	dlls of a different version than what was supplied with this program then you have to know what pixel formats
	are valid for that version of the dlls since valid pixel formats changes between versions.
@param ePixFmt the pixel format which we want to describe.
@param psPixDesc the structure that will recieve the description of the pixel format. */
extern "C" FFMPEGINTERFACE_API void __stdcall FFPixDesc(PixelFormat ePixFmt, AVPixFmtDescriptor* psPixDesc);
#endif